@@ -22,25 +22,25 @@
     without including the source code for Qt in the source distribution.
 */
 
-#ifndef ABOUTDATA_H
-#define ABOUTDATA_H
+#ifndef AKREGATOR_ABOUTDATA_H
+#define AKREGATOR_ABOUTDATA_H
 
+#include "akregator_export.h"
 #include <kaboutdata.h>
-#include <kdepimmacros.h>
 
-#define AKREGATOR_VERSION "1.2.8"
+#define AKREGATOR_VERSION "1.2.50"
 
 namespace Akregator {
 /**
 @author Teemu Rytilahti
 */
-class KDE_EXPORT AboutData : public KAboutData
+class AKREGATOR_EXPORT AboutData : public KAboutData
 {
 public:
     AboutData();
     ~AboutData();
 };
 
-}
+} // namespace Akregator
 
-#endif
+#endif // AKREGATOR_ABOUTDATA_H
