@@ -2,7 +2,7 @@
     This file is part of Akregator.
 
     Copyright (C) 2004 Stanislav Karchebny <Stanislav.Karchebny@kdemail.net>
-                  2005 Frank Osterfeld <frank.osterfeld at kdemail.net>
+                  2005 Frank Osterfeld <osterfeld@kde.org>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -23,38 +23,49 @@
     without including the source code for Qt in the source distribution.
 */
 
-#include <qtimer.h>
-#include <qdatetime.h>
-#include <qlistview.h>
-#include <qdom.h>
-#include <qmap.h>
-#include <qpixmap.h>
-#include <qvaluelist.h>
-
-#include <kurl.h>
-#include <kdebug.h>
-#include <kglobal.h>
-#include <kstandarddirs.h>
-
 #include "akregatorconfig.h"
 #include "article.h"
-#include "articleinterceptor.h"
+#include "articlejobs.h"
 #include "feed.h"
-#include "folder.h"
-#include "fetchqueue.h"
 #include "feediconmanager.h"
 #include "feedstorage.h"
+#include "fetchqueue.h"
+#include "folder.h"
 #include "storage.h"
 #include "treenodevisitor.h"
+#include "types.h"
 #include "utils.h"
+#include <syndication/syndication.h>
 
-#include "librss/librss.h"
+#include <kdebug.h>
+#include <kglobal.h>
+#include <KIcon>
+#include <kstandarddirs.h>
+#include <kurl.h>
 
-namespace Akregator {
+//#include <qtl.h>
 
-class Feed::FeedPrivate
+#include <QDateTime>
+#include <QDomDocument>
+#include <QDomElement>
+#include <QHash>
+#include <QIcon>
+#include <QList>
+#include <QPixmap>
+
+#include <boost/bind.hpp>
+
+using Syndication::ItemPtr;
+using namespace Akregator;
+using namespace boost;
+
+class Feed::Private
 {
+        Feed* const q;
     public:
+        explicit Private( Akregator::Backend::Storage* storage, Feed* qq );
+        
+        Akregator::Backend::Storage* storage;
         bool autoFetch;
         int fetchInterval;
         ArchiveMode archiveMode;
@@ -63,17 +74,12 @@ class Feed::FeedPrivate
         bool markImmediatelyAsRead;
         bool useNotification;
         bool loadLinkedWebsite;
+        int lastFetched;
 
-        bool fetchError;
-        
-        int lastErrorFetch; // save time of last fetch that went wrong.
-                            // != lastFetch property from the archive 
-                            // (that saves the last _successfull fetch!)
-                            // workaround for 3.5.x
-
+        Syndication::ErrorCode fetchErrorCode;
         int fetchTries;
         bool followDiscovery;
-        RSS::Loader* loader;
+        Syndication::Loader* loader;
         bool articlesLoaded;
         Backend::FeedStorage* archive;
 
@@ -82,25 +88,22 @@ class Feed::FeedPrivate
         QString description;
 
         /** list of feed articles */
-        QMap<QString, Article> articles;
-
-        /** caches guids of tagged articles. key: tag, value: list of guids */
-        QMap<QString, QStringList> taggedArticles;
+        QHash<QString, Article> articles;
 
         /** list of deleted articles. This contains **/
-        QValueList<Article> deletedArticles;
-        
+        QList<Article> deletedArticles;
+
         /** caches guids of deleted articles for notification */
-   
-        QValueList<Article> addedArticlesNotify;
-        QValueList<Article> removedArticlesNotify;
-        QValueList<Article> updatedArticlesNotify;
-        
+
+        QList<Article> addedArticlesNotify;
+        QList<Article> removedArticlesNotify;
+        QList<Article> updatedArticlesNotify;
+
         QPixmap imagePixmap;
-        RSS::Image image;
-        QPixmap favicon;
+        Syndication::ImagePtr image;
+        QIcon favicon;
 };
-            
+
 QString Feed::archiveModeToString(ArchiveMode mode)
 {
     switch (mode)
@@ -122,51 +125,46 @@ QString Feed::archiveModeToString(ArchiveMode mode)
    return "globalDefault";
 }
 
-Feed* Feed::fromOPML(QDomElement e)
+Feed* Feed::fromOPML(QDomElement e, Akregator::Backend::Storage* storage )
 {
 
-    Feed* feed = 0;
+    if( !e.hasAttribute("xmlUrl") && !e.hasAttribute("xmlurl") && !e.hasAttribute("xmlURL") )
+        return 0;
+    
+    QString title = e.hasAttribute("text") ? e.attribute("text") : e.attribute("title");
 
-    if( e.hasAttribute("xmlUrl") || e.hasAttribute("xmlurl") || e.hasAttribute("xmlURL") )
-    {
-        QString title = e.hasAttribute("text") ? e.attribute("text") : e.attribute("title");
+    QString xmlUrl = e.hasAttribute("xmlUrl") ? e.attribute("xmlUrl") : e.attribute("xmlurl");
+    if (xmlUrl.isEmpty())
+        xmlUrl = e.attribute("xmlURL");
 
-        QString xmlUrl = e.hasAttribute("xmlUrl") ? e.attribute("xmlUrl") : e.attribute("xmlurl");
-        if (xmlUrl.isEmpty())
-            xmlUrl = e.attribute("xmlURL");
+    bool useCustomFetchInterval = e.attribute("useCustomFetchInterval") == "true";
 
-        bool useCustomFetchInterval = e.attribute("useCustomFetchInterval") == "true" || e.attribute("autoFetch") == "true"; 
-        // "autoFetch" is used in 3.4
-        // Will be removed in KDE4
+    QString htmlUrl = e.attribute("htmlUrl");
+    QString description = e.attribute("description");
+    int fetchInterval = e.attribute("fetchInterval").toInt();
+    ArchiveMode archiveMode = stringToArchiveMode(e.attribute("archiveMode"));
+    int maxArticleAge = e.attribute("maxArticleAge").toUInt();
+    int maxArticleNumber = e.attribute("maxArticleNumber").toUInt();
+    bool markImmediatelyAsRead = e.attribute("markImmediatelyAsRead") == "true";
+    bool useNotification = e.attribute("useNotification") == "true";
+    bool loadLinkedWebsite = e.attribute("loadLinkedWebsite") == "true";
+    uint id = e.attribute("id").toUInt();
 
-        QString htmlUrl = e.attribute("htmlUrl");
-        QString description = e.attribute("description");
-        int fetchInterval = e.attribute("fetchInterval").toInt();
-        ArchiveMode archiveMode = stringToArchiveMode(e.attribute("archiveMode"));
-        int maxArticleAge = e.attribute("maxArticleAge").toUInt();
-        int maxArticleNumber = e.attribute("maxArticleNumber").toUInt();
-        bool markImmediatelyAsRead = e.attribute("markImmediatelyAsRead") == "true";
-        bool useNotification = e.attribute("useNotification") == "true";
-        bool loadLinkedWebsite = e.attribute("loadLinkedWebsite") == "true";
-        uint id = e.attribute("id").toUInt();
-
-        feed = new Feed();
-        feed->setTitle(title);
-        feed->setXmlUrl(xmlUrl);
-        feed->setCustomFetchIntervalEnabled(useCustomFetchInterval);
-        feed->setHtmlUrl(htmlUrl);
-        feed->setId(id);
-        feed->setDescription(description);
-        feed->setArchiveMode(archiveMode);
-        feed->setUseNotification(useNotification);
-        feed->setFetchInterval(fetchInterval);
-        feed->setMaxArticleAge(maxArticleAge);
-        feed->setMaxArticleNumber(maxArticleNumber);
-        feed->setMarkImmediatelyAsRead(markImmediatelyAsRead);
-        feed->setLoadLinkedWebsite(loadLinkedWebsite);
-        feed->loadArticles(); // TODO: make me fly: make this delayed
-        feed->loadImage();
-    }
+    Feed* const feed = new Feed( storage );
+    feed->setTitle(title);
+    feed->setXmlUrl(xmlUrl);
+    feed->setCustomFetchIntervalEnabled(useCustomFetchInterval);
+    feed->setHtmlUrl(htmlUrl);
+    feed->setId(id);
+    feed->setDescription(description);
+    feed->setArchiveMode(archiveMode);
+    feed->setUseNotification(useNotification);
+    feed->setFetchInterval(fetchInterval);
+    feed->setMaxArticleAge(maxArticleAge);
+    feed->setMaxArticleNumber(maxArticleNumber);
+    feed->setMarkImmediatelyAsRead(markImmediatelyAsRead);
+    feed->setLoadLinkedWebsite(loadLinkedWebsite);
+    feed->loadArticles(); // TODO: make me fly: make this delayed
 
     return feed;
 }
@@ -179,9 +177,28 @@ bool Feed::accept(TreeNodeVisitor* visitor)
         return visitor->visitTreeNode(this);
 }
 
-QStringList Feed::tags() const
+QVector<const Folder*> Feed::folders() const
 {
-    return d->archive->tags();
+    return QVector<const Folder*>();
+}
+
+QVector<Folder*> Feed::folders()
+{
+    return QVector<Folder*>();
+}
+
+QVector<const Feed*> Feed::feeds() const
+{
+    QVector<const Feed*> list;
+    list.append( this );
+    return list;
+}
+
+QVector<Feed*> Feed::feeds()
+{
+    QVector<Feed*> list;
+    list.append( this );
+    return list;
 }
 
 Article Feed::findArticle(const QString& guid) const
@@ -189,38 +206,25 @@ Article Feed::findArticle(const QString& guid) const
     return d->articles[guid];
 }
 
-QValueList<Article> Feed::articles(const QString& tag)
+QList<Article> Feed::articles()
 {
     if (!d->articlesLoaded)
         loadArticles();
-    if (tag.isNull())
-        return d->articles.values();
-    else
-    {
-        QValueList<Article> tagged;
-        QStringList guids = d->archive->articles(tag);
-        for (QStringList::ConstIterator it = guids.begin(); it != guids.end(); ++it)
-            tagged += d->articles[*it];
-        return tagged;
-        
-    }
+    return d->articles.values();
 }
 
-void Feed::loadImage()
+Akregator::Backend::Storage* Feed::storage()
 {
-    QString imageFileName = KGlobal::dirs()->saveLocation("cache", "akregator/Media/") 
-                            + Utils::fileNameForUrl(d->xmlUrl) + 
-".png";
-    d->imagePixmap.load(imageFileName, "PNG");
+    return d->storage;
 }
-        
+
 void Feed::loadArticles()
 {
     if (d->articlesLoaded)
         return;
 
     if (!d->archive)
-        d->archive = Backend::Storage::getInstance()->archiveFor(xmlUrl());
+        d->archive = d->storage->archiveFor(xmlUrl());
 
     QStringList list = d->archive->articles();
     for ( QStringList::ConstIterator it = list.begin(); it != list.end(); ++it)
@@ -230,7 +234,7 @@ void Feed::loadArticles()
         if (mya.isDeleted())
             d->deletedArticles.append(mya);
     }
-    
+
     d->articlesLoaded = true;
     enforceLimitArticleNumber();
     recalcUnreadCount();
@@ -238,16 +242,16 @@ void Feed::loadArticles()
 
 void Feed::recalcUnreadCount()
 {
-    QValueList<Article> tarticles = articles();
-    QValueList<Article>::Iterator it;
-    QValueList<Article>::Iterator en = tarticles.end();
+    QList<Article> tarticles = articles();
+    QList<Article>::Iterator it;
+    QList<Article>::Iterator en = tarticles.end();
 
     int oldUnread = d->archive->unread();
-    
+
     int unread = 0;
 
     for (it = tarticles.begin(); it != en; ++it)
-        if (!(*it).isDeleted() && (*it).status() != Article::Read)
+        if (!(*it).isDeleted() && (*it).status() != Read)
             ++unread;
 
     if (unread != oldUnread)
@@ -273,26 +277,36 @@ Feed::ArchiveMode Feed::stringToArchiveMode(const QString& str)
     return globalDefault;
 }
 
-Feed::Feed() : TreeNode(), d(new FeedPrivate)
+Feed::Private::Private( Akregator::Backend::Storage* storage_, Feed* qq ) 
+  : q( qq ),
+    storage( storage_ ),
+    autoFetch( false ),
+    fetchInterval( 30 ),
+    archiveMode( globalDefault ),
+    maxArticleAge( 60 ),
+    maxArticleNumber( 1000 ),
+    markImmediatelyAsRead( false ),
+    useNotification( false ),
+    loadLinkedWebsite( false ),
+    lastFetched( 0 ),
+    fetchErrorCode( Syndication::Success ),
+    fetchTries( 0 ),
+    followDiscovery( false ),
+    loader( 0 ),
+    articlesLoaded( false ),
+    archive( 0 )
+{
+    assert( q );
+    assert( storage );
+}
+
+Feed::Feed( Akregator::Backend::Storage* storage ) : TreeNode(), d( new Private( storage, this ) )
 {
-    d->autoFetch = false;
-    d->fetchInterval = 30;
-    d->archiveMode = globalDefault;
-    d->maxArticleAge = 60;
-    d->maxArticleNumber = 1000;
-    d->markImmediatelyAsRead = false;
-    d->useNotification = false;
-    d->fetchError = false;
-    d->lastErrorFetch = 0;
-    d->fetchTries = 0;
-    d->loader = 0;
-    d->articlesLoaded = false;
-    d->archive = 0;
-    d->loadLinkedWebsite = false;
 }
 
 Feed::~Feed()
 {
+    FeedIconManager::self()->removeListener( this );
     slotAbortFetch();
     emitSignalDestroyed();
     delete d;
@@ -317,6 +331,8 @@ void Feed::setMaxArticleNumber(int maxArticleNumber) { d->maxArticleNumber = max
 
 bool Feed::markImmediatelyAsRead() const { return d->markImmediatelyAsRead; }
 
+bool Feed::isFetching() const { return d->loader != 0; }
+
 void Feed::setMarkImmediatelyAsRead(bool enabled)
 {
     d->markImmediatelyAsRead = enabled;
@@ -343,24 +359,27 @@ bool Feed::loadLinkedWebsite() const
 {
     return d->loadLinkedWebsite;
 }
-            
-const QPixmap& Feed::favicon() const { return d->favicon; }
 
-const QPixmap& Feed::image() const { return d->imagePixmap; }
+QPixmap Feed::image() const { return d->imagePixmap; }
 
-const QString& Feed::xmlUrl() const { return d->xmlUrl; }
+QString Feed::xmlUrl() const { return d->xmlUrl; }
 
-void Feed::setXmlUrl(const QString& s) { d->xmlUrl = s; }
+void Feed::setXmlUrl(const QString& s)
+{
+    d->xmlUrl = s;
+}
 
-const QString& Feed::htmlUrl() const { return d->htmlUrl; }
+QString Feed::htmlUrl() const { return d->htmlUrl; }
 
 void Feed::setHtmlUrl(const QString& s) { d->htmlUrl = s; }
 
-const QString& Feed::description() const { return d->description; }
+QString Feed::description() const { return d->description; }
 
 void Feed::setDescription(const QString& s) { d->description = s; }
 
-bool Feed::fetchErrorOccurred() { return d->fetchError; }
+bool Feed::fetchErrorOccurred() const { return d->fetchErrorCode != Syndication::Success; }
+
+Syndication::ErrorCode Feed::fetchErrorCode() const { return d->fetchErrorCode; }
 
 bool Feed::isArticlesLoaded() const { return d->articlesLoaded; }
 
@@ -396,14 +415,13 @@ void Feed::slotMarkAllArticlesAsRead()
 {
     if (unread() > 0)
     {
-        setNotificationMode(false, true);
-        QValueList<Article> tarticles = articles();
-        QValueList<Article>::Iterator it;
-        QValueList<Article>::Iterator en = tarticles.end();
-
-        for (it = tarticles.begin(); it != en; ++it)
-            (*it).setStatus(Article::Read);
-        setNotificationMode(true, true);
+        Akregator::ArticleModifyJob* job = new Akregator::ArticleModifyJob;
+        Q_FOREACH ( const Akregator::Article i, articles() )    
+        {
+            const Akregator::ArticleId aid = { xmlUrl(), i.guid() };
+            job->setStatus( aid, Akregator::Read );
+        }
+        job->start();
     }
 }
 void Feed::slotAddToFetchQueue(FetchQueue* queue, bool intervalFetchOnly)
@@ -412,14 +430,6 @@ void Feed::slotAddToFetchQueue(FetchQueue* queue, bool intervalFetchOnly)
         queue->addFeed(this);
     else
     {
-        uint now = QDateTime::currentDateTime().toTime_t();
-
-        // workaround for 3.5.x: if the last fetch went wrong, try again after 30 minutes
-        // this fixes annoying behaviour of akregator, especially when the host is reachable
-        // but Akregator can't parse the feed (the host is hammered every minute then)
-        if ( fetchErrorOccurred() && now - d->lastErrorFetch <= 30*60 )
-             return;
-
         int interval = -1;
 
         if (useCustomFetchInterval() )
@@ -430,56 +440,54 @@ void Feed::slotAddToFetchQueue(FetchQueue* queue, bool intervalFetchOnly)
 
         uint lastFetch = d->archive->lastFetch();
 
+        uint now = QDateTime::currentDateTime().toTime_t();
+
         if ( interval > 0 && now - lastFetch >= (uint)interval )
             queue->addFeed(this);
     }
 }
 
 
-void Feed::appendArticles(const RSS::Document &doc)
+void Feed::appendArticles(const Syndication::FeedPtr feed)
 {
     bool changed = false;
 
-    RSS::Article::List d_articles = doc.articles();
-    RSS::Article::List::ConstIterator it;
-    RSS::Article::List::ConstIterator en = d_articles.end();
+    QList<ItemPtr> items = feed->items();
+    QList<ItemPtr>::ConstIterator it = items.begin();
+    QList<ItemPtr>::ConstIterator en = items.end();
+
 
     int nudge=0;
-    
-    QValueList<Article> deletedArticles = d->deletedArticles;
 
-    for (it = d_articles.begin(); it != en; ++it)
+    QList<Article> deletedArticles = d->deletedArticles;
+
+    for ( ; it != en; ++it)
     {
-        if ( !d->articles.contains((*it).guid()) ) // article not in list
+        if ( !d->articles.contains((*it)->id()) ) // article not in list
         {
             Article mya(*it, this);
             mya.offsetPubDate(nudge);
             nudge--;
             appendArticle(mya);
-
-            QValueList<ArticleInterceptor*> interceptors = ArticleInterceptorManager::self()->interceptors();
-            for (QValueList<ArticleInterceptor*>::ConstIterator it = interceptors.begin(); it != interceptors.end(); ++it)
-                (*it)->processArticle(mya);
-            
             d->addedArticlesNotify.append(mya);
-            
+
             if (!mya.isDeleted() && !markImmediatelyAsRead())
-                mya.setStatus(Article::New);
+                mya.setStatus(New);
             else
-                mya.setStatus(Article::Read);
-                
+                mya.setStatus(Read);
+
             changed = true;
         }
         else // article is in list
         {
             // if the article's guid is no hash but an ID, we have to check if the article was updated. That's done by comparing the hash values.
-            Article old = d->articles[(*it).guid()];
-            Article mya(*it, this);          
+            Article old = d->articles[(*it)->id()];
+            Article mya(*it, this);
             if (!mya.guidIsHash() && mya.hash() != old.hash() && !old.isDeleted())
             {
                 mya.setKeep(old.keep());
                 int oldstatus = old.status();
-                old.setStatus(Article::Read);
+                old.setStatus(Read);
 
                 d->articles.remove(old.guid());
                 appendArticle(mya);
@@ -490,13 +498,13 @@ void Feed::appendArticles(const RSS::Document &doc)
                 changed = true;
             }
             else if (old.isDeleted())
-                deletedArticles.remove(mya);
-        }    
+                deletedArticles.removeAll(mya);
+        }
     }
-    
-    QValueList<Article>::ConstIterator dit = deletedArticles.begin();
-    QValueList<Article>::ConstIterator dtmp;
-    QValueList<Article>::ConstIterator den = deletedArticles.end();
+
+    QList<Article>::ConstIterator dit = deletedArticles.begin();
+    QList<Article>::ConstIterator dtmp;
+    QList<Article>::ConstIterator den = deletedArticles.end();
 
     // delete articles with delete flag set completely from archive, which aren't in the current feed source anymore
     while (dit != den)
@@ -505,9 +513,11 @@ void Feed::appendArticles(const RSS::Document &doc)
         ++dit;
         d->articles.remove((*dtmp).guid());
         d->archive->deleteArticle((*dtmp).guid());
-        d->deletedArticles.remove(*dtmp);
+        d->removedArticlesNotify.append( *dtmp );
+        changed = true;
+        d->deletedArticles.removeAll(*dtmp);
     }
-    
+
     if (changed)
         articlesModified();
 }
@@ -538,7 +548,7 @@ void Feed::appendArticle(const Article& a)
         if (!d->articles.contains(a.guid()))
         {
             d->articles[a.guid()] = a;
-            if (!a.isDeleted() && a.status() != Article::Read)
+            if (!a.isDeleted() && a.status() != Read)
                 setUnread(unread()+1);
         }
     }
@@ -551,14 +561,14 @@ void Feed::fetch(bool followDiscovery)
     d->fetchTries = 0;
 
     // mark all new as unread
-    QValueList<Article> articles = d->articles.values();
-    QValueList<Article>::Iterator it;
-    QValueList<Article>::Iterator en = articles.end();
+    QList<Article> articles = d->articles.values();
+    QList<Article>::Iterator it;
+    QList<Article>::Iterator en = articles.end();
     for (it = articles.begin(); it != en; ++it)
     {
-        if ((*it).status() == Article::New)
+        if ((*it).status() == New)
         {
-            (*it).setStatus(Article::Unread);
+            (*it).setStatus(Unread);
         }
     }
 
@@ -577,38 +587,34 @@ void Feed::slotAbortFetch()
 
 void Feed::tryFetch()
 {
-    d->fetchError = false;
+    d->fetchErrorCode = Syndication::Success;
 
-    d->loader = RSS::Loader::create( this, SLOT(fetchCompleted(Loader *, Document, Status)) );
+    d->loader = Syndication::Loader::create( this, SLOT(fetchCompleted(Syndication::Loader*,
+                                                                       Syndication::FeedPtr,
+                                                                       Syndication::ErrorCode)) );
     //connect(d->loader, SIGNAL(progress(unsigned long)), this, SLOT(slotSetProgress(unsigned long)));
-    d->loader->loadFrom( d->xmlUrl, new RSS::FileRetriever );
+    d->loader->loadFrom( d->xmlUrl);
 }
 
 void Feed::slotImageFetched(const QPixmap& image)
 {
-    if (image.isNull())
-        return;
-    d->imagePixmap=image;
-    d->imagePixmap.save(KGlobal::dirs()->saveLocation("cache", "akregator/Media/") 
-                        + Utils::fileNameForUrl(d->xmlUrl) + 
-".png","PNG");
-    nodeModified();
+    setImage(image);
 }
 
-void Feed::fetchCompleted(RSS::Loader *l, RSS::Document doc, RSS::Status status)
+void Feed::fetchCompleted(Syndication::Loader *l, Syndication::FeedPtr doc, Syndication::ErrorCode status)
 {
     // Note that loader instances delete themselves
     d->loader = 0;
 
     // fetching wasn't successful:
-    if (status != RSS::Success)
+    if (status != Syndication::Success)
     {
-        if (status == RSS::Aborted)
+        if (status == Syndication::Aborted)
         {
-            d->fetchError = false;
+            d->fetchErrorCode = Syndication::Success;
             emit fetchAborted(this);
         }
-        else if (d->followDiscovery && (status == RSS::ParseError) && (d->fetchTries < 3) && (l->discoveredFeedURL().isValid()))
+        else if (d->followDiscovery && (status == Syndication::InvalidXml) && (d->fetchTries < 3) && (l->discoveredFeedURL().isValid()))
         {
             d->fetchTries++;
             d->xmlUrl = l->discoveredFeedURL().url();
@@ -617,33 +623,40 @@ void Feed::fetchCompleted(RSS::Loader *l, RSS::Document doc, RSS::Status status)
         }
         else
         {
-            d->fetchError = true;
-            d->lastErrorFetch = QDateTime::currentDateTime().toTime_t();
+            d->fetchErrorCode = status;
             emit fetchError(this);
         }
         return;
     }
 
     loadArticles(); // TODO: make me fly: make this delayed
-    
-    // Restore favicon.
-    if (d->favicon.isNull())
-        loadFavicon();
 
-    d->fetchError = false;
+    FeedIconManager::self()->addListener( KUrl( xmlUrl() ), this );
     
-    if (doc.image() && d->imagePixmap.isNull())
+    d->fetchErrorCode = Syndication::Success;
+
+    if (d->imagePixmap.isNull())
     {
-        d->image = *doc.image();
-        connect(&d->image, SIGNAL(gotPixmap(const QPixmap&)), this, SLOT(slotImageFetched(const QPixmap&)));
-        d->image.getPixmap();
+        QString u = d->xmlUrl;
+        QString imageFileName = KGlobal::dirs()->saveLocation("cache", "akregator/Media/")
+                                + Utils::fileNameForUrl(d->xmlUrl) + ".png";
+        d->imagePixmap=QPixmap(imageFileName, "PNG");
+
+        // if we aint got teh image and the feed provides one, get it....
+        // TODO: reenable image fetching!
+        if (false) // d->imagePixmap.isNull() && doc.image())
+        {
+            //d->image = *doc.image();
+            //connect(&d->image, SIGNAL(gotPixmap(const QPixmap&)), this, SLOT(slotImageFetched(const QPixmap&)));
+            //d->image.getPixmap();
+        }
     }
 
     if (title().isEmpty())
-        setTitle( doc.title() );
+        setTitle( doc->title() );
 
-    d->description = doc.description();
-    d->htmlUrl = doc.link().url();
+    d->description = doc->description();
+    d->htmlUrl = doc->link();
 
     appendArticles(doc);
 
@@ -651,51 +664,51 @@ void Feed::fetchCompleted(RSS::Loader *l, RSS::Document doc, RSS::Status status)
     emit fetched(this);
 }
 
-void Feed::loadFavicon()
+QIcon Feed::icon() const
 {
-    FeedIconManager::self()->fetchIcon(this);
+    if ( fetchErrorOccurred() )
+        return KIcon("dialog-error");
+
+    return !d->favicon.isNull() ? d->favicon : KIcon("text-html");
 }
 
-void Feed::slotDeleteExpiredArticles()
+void Feed::deleteExpiredArticles( Akregator::ArticleDeleteJob* deleteJob )
 {
     if ( !usesExpiryByAge() )
         return;
 
-    QValueList<Article> articles = d->articles.values();
-    
-    QValueList<Article>::Iterator en = articles.end();
-
     setNotificationMode(false);
 
-    // check keep flag only if it should be respected for expiry
-    // the code could be more compact, but we better check
-    // doNotExpiredArticles once instead of in every iteration
-    if (Settings::doNotExpireImportantArticles())
+    const QList<Article> articles = d->articles.values();
+    QList<Akregator::ArticleId> toDelete;
+    const QString feedUrl = xmlUrl();
+    const bool useKeep = Settings::doNotExpireImportantArticles();
+
+    Q_FOREACH ( const Akregator::Article i, articles )
     {
-        for (QValueList<Article>::Iterator it = articles.begin(); it != en; ++it)
+        if ( ( !useKeep || !i.keep() ) && isExpired( i ) )
         {
-            if (!(*it).keep() && isExpired(*it))
-            {
-                    (*it).setDeleted();
-            }
-        }
-    }
-    else
-    {
-        for (QValueList<Article>::Iterator it = articles.begin(); it != en; ++it)
-        {
-            if (isExpired(*it))
-            {
-                    (*it).setDeleted();
-            }
+            const ArticleId aid = { feedUrl, i.guid() };
+            toDelete.append( aid );
         }
     }
+
+    deleteJob->appendArticleIds( toDelete );
     setNotificationMode(true);
 }
 
-void Feed::setFavicon(const QPixmap &p)
+void Feed::setFavicon( const QIcon& icon )
 {
-    d->favicon = p;
+    d->favicon = icon;
+    nodeModified();
+}
+
+void Feed::setImage(const QPixmap &p)
+{
+    if (p.isNull())
+        return;
+    d->imagePixmap=p;
+    d->imagePixmap.save(KGlobal::dirs()->saveLocation("cache", "akregator/Media/")+ Utils::fileNameForUrl(d->xmlUrl) + ".png","PNG");
     nodeModified();
 }
 
@@ -729,8 +742,7 @@ void Feed::setArticleDeleted(Article& a)
     if (!d->deletedArticles.contains(a))
         d->deletedArticles.append(a);
 
-    if (!d->removedArticlesNotify.contains(a))
-        d->removedArticlesNotify.append(a);
+    d->updatedArticlesNotify.append(a);
 
     articlesModified();
 }
@@ -740,9 +752,9 @@ void Feed::setArticleChanged(Article& a, int oldStatus)
     if (oldStatus != -1)
     {
         int newStatus = a.status();
-        if (oldStatus == Article::Read && newStatus != Article::Read)
+        if (oldStatus == Read && newStatus != Read)
             setUnread(unread()+1);
-        else if (oldStatus != Article::Read && newStatus == Article::Read)
+        else if (oldStatus != Read && newStatus == Read)
             setUnread(unread()-1);
     }
     d->updatedArticlesNotify.append(a);
@@ -751,7 +763,7 @@ void Feed::setArticleChanged(Article& a, int oldStatus)
 
 int Feed::totalCount() const
 {
-    return d->articles.count();
+    return std::count_if( d->articles.begin(), d->articles.end(), !bind( &Article::isDeleted, _1 ) );
 }
 
 TreeNode* Feed::next()
@@ -770,13 +782,30 @@ TreeNode* Feed::next()
     return 0;
 }
 
+
+const TreeNode* Feed::next() const
+{
+    if ( nextSibling() )
+        return nextSibling();
+
+    const Folder* p = parent();
+    while (p)
+    {
+        if ( p->nextSibling() )
+            return p->nextSibling();
+        else
+            p = p->parent();
+    }
+    return 0;
+}
+
 void Feed::doArticleNotification()
 {
     if (!d->addedArticlesNotify.isEmpty())
     {
-        // copy list, otherwise the refcounting in Article::Private breaks for 
+        // copy list, otherwise the refcounting in Article::Private breaks for
         // some reason (causing segfaults)
-        QValueList<Article> l = d->addedArticlesNotify;
+        QList<Article> l = d->addedArticlesNotify;
         emit signalArticlesAdded(this, l);
         d->addedArticlesNotify.clear();
     }
@@ -784,15 +813,15 @@ void Feed::doArticleNotification()
     {
         // copy list, otherwise the refcounting in Article::Private breaks for
         // some reason (causing segfaults)
-        QValueList<Article> l = d->updatedArticlesNotify;
+        QList<Article> l = d->updatedArticlesNotify;
         emit signalArticlesUpdated(this, l);
         d->updatedArticlesNotify.clear();
     }
     if (!d->removedArticlesNotify.isEmpty())
     {
-        // copy list, otherwise the refcounting in Article::Private breaks for 
+        // copy list, otherwise the refcounting in Article::Private breaks for
         // some reason (causing segfaults)
-        QValueList<Article> l = d->removedArticlesNotify;
+        QList<Article> l = d->removedArticlesNotify;
         emit signalArticlesRemoved(this, l);
         d->removedArticlesNotify.clear();
     }
@@ -806,52 +835,32 @@ void Feed::enforceLimitArticleNumber()
         limit = Settings::maxArticleNumber();
     else if (d->archiveMode == limitArticleNumber)
         limit = maxArticleNumber();
-        
+
     if (limit == -1 || limit >= d->articles.count() - d->deletedArticles.count())
         return;
 
-    setNotificationMode(false);
-    QValueList<Article> articles = d->articles.values();
-    qHeapSort(articles);
-    QValueList<Article>::Iterator it = articles.begin();
-    QValueList<Article>::Iterator tmp;
-    QValueList<Article>::Iterator en = articles.end();
+    QList<Article> articles = d->articles.values();
+    qSort(articles);
 
     int c = 0;
-    
-    if (Settings::doNotExpireImportantArticles())
+    const bool useKeep = Settings::doNotExpireImportantArticles();
+
+    ArticleDeleteJob* job = new ArticleDeleteJob;
+    Q_FOREACH ( const Akregator::Article i, articles )
     {
-        while (it != en)
+        if (c < limit)
         {
-            tmp = it;
-            ++it;
-            if (c < limit)
-            {
-                if (!(*tmp).isDeleted() && !(*tmp).keep())
-                c++;
-            }
-            else if (!(*tmp).keep())
-                (*tmp).setDeleted();
+            if ( !i.isDeleted() && ( !useKeep || !i.keep() ) )
+                ++c;
+        }
+        else if ( !useKeep || !i.keep() )
+        {
+            const ArticleId aid = { i.feed()->xmlUrl(), i.guid() };
+            job->appendArticleId( aid );
         }
     }
-    else
-    {
-        while (it != en)
-        {
-            tmp = it;
-            ++it;
-            if (c < limit && !(*tmp).isDeleted())
-            {
-                c++;
-            }
-            else
-            {
-                (*tmp).setDeleted();
-            }
-        }
-    }
-    setNotificationMode(true);
+    job->start();
 }
 
-} // namespace Akregator
 #include "feed.moc"
+    
