@@ -28,16 +28,15 @@
 
 //settings
 
-#include <dcopclient.h>
-
 #include <kaction.h>
+#include <kactioncollection.h>
 #include <kapplication.h>
 #include <kconfig.h>
 #include <kdebug.h>
 #include <kedittoolbar.h>
 #include <kfiledialog.h>
 #include <kglobal.h>
-#include <kkeydialog.h>
+#include <kshortcutsdialog.h>
 #include <klibloader.h>
 #include <klocale.h>
 #include <kmessagebox.h>
@@ -45,31 +44,34 @@
 #include <ksqueezedtextlabel.h>
 #include <kstandarddirs.h>
 #include <kstatusbar.h>
-#include <kstdaction.h>
+#include <kstandardaction.h>
+#include <ktoolbar.h>
 #include <kurl.h>
 
 #include "progressdialog.h"
 #include "statusbarprogresswidget.h"
 #include "trayicon.h"
 
-#include <qmetaobject.h>
-#include <qpen.h>
-#include <qpainter.h>
-#include <private/qucomextra_p.h>
-#include <qtimer.h>
+#include <QPixmap>
 
+#include <Q3MimeSourceFactory>
 
 namespace Akregator {
 
 BrowserInterface::BrowserInterface( MainWindow *shell, const char *name )
-    : KParts::BrowserInterface( shell, name )
+    : KParts::BrowserInterface( shell)
 {
+    setObjectName(name);
     m_shell = shell;
 }
 
 MainWindow::MainWindow()
-    : KParts::MainWindow( 0L, "akregator_mainwindow" ){
+    : KParts::MainWindow()
+{
+    setPluginLoadingMode( DoNotLoadPlugins );
+
     // set the shell's ui resource file
+    setObjectName("akregator_mainwindow");
     setXMLFile("akregator_shell.rc");
 
     m_browserIface=new BrowserInterface(this, "browser_interface");
@@ -88,7 +90,7 @@ MainWindow::MainWindow()
     m_statusLabel->setSizePolicy(QSizePolicy( QSizePolicy::Ignored, QSizePolicy::Fixed ));
     m_statusLabel->setMinimumWidth( 0 );
     m_statusLabel->setFixedHeight( statH );
-    statusBar()->addWidget (m_statusLabel, 1, false);
+    statusBar()->addWidget( m_statusLabel, 1 );
 
     setupActions();
     createGUI(0L);
@@ -99,15 +101,16 @@ bool MainWindow::loadPart()
     // this routine will find and load our Part.  it finds the Part by
     // name which is a bad idea usually.. but it's alright in this
     // case since our Part is made for this Shell
-    KLibFactory *factory = KLibLoader::self()->factory("libakregatorpart");
+    KPluginFactory *factory = KPluginLoader("akregatorpart").factory();
     if (factory)
     {
         // now that the Part is loaded, we cast it to a Part to get
         // our hands on it
-        m_part = static_cast<Akregator::Part*>(factory->create(this, "akregator_part", "KParts::ReadOnlyPart" ));
+        m_part = static_cast<Akregator::Part*>( factory->create<KParts::ReadOnlyPart>(this) );
 
         if (m_part)
         {
+            m_part->setObjectName("akregator_part");
             // tell the KParts::MainWindow that this is indeed the main widget
             setCentralWidget(m_part->widget());
 
@@ -115,7 +118,6 @@ bool MainWindow::loadPart()
 
             connect(TrayIcon::getInstance(), SIGNAL(quitSelected()), this, SLOT(slotQuit()));
             // and integrate the part's GUI with the shell's
-            connectActionCollection(m_part->actionCollection());
             createGUI(m_part);
             browserExtension(m_part)->setBrowserInterface(m_browserIface);
             setAutoSaveSettings();
@@ -138,7 +140,7 @@ void MainWindow::setupProgressWidgets()
     progressDialog->hide();
     m_progressBar = new KPIM::StatusbarProgressWidget( progressDialog, statusBar() );
     m_progressBar->show();
-    statusBar()->addWidget( m_progressBar, 0, true );
+    statusBar()->addPermanentWidget( m_progressBar, 0 );
 }
 
 MainWindow::~MainWindow()
@@ -152,35 +154,33 @@ void MainWindow::setCaption(const QString &a)
 
 void MainWindow::setupActions()
 {
-    connectActionCollection(actionCollection());
-
-    KStdAction::quit(kapp, SLOT(quit()), actionCollection());
+    KStandardAction::quit(kapp, SLOT(quit()), actionCollection());
 
     setStandardToolBarMenuEnabled(true);
     createStandardStatusBarAction();
 
-    KStdAction::keyBindings(this, SLOT(optionsConfigureKeys()), actionCollection());
-    KStdAction::configureToolbars(this, SLOT(optionsConfigureToolbars()), actionCollection());
+    KStandardAction::keyBindings(this, SLOT(optionsConfigureKeys()), actionCollection());
+    KStandardAction::configureToolbars(this, SLOT(optionsConfigureToolbars()), actionCollection());
 }
 
-void MainWindow::saveProperties(KConfig* config)
+void MainWindow::saveProperties(KConfigGroup &config)
 {
     if (!m_part)
         loadPart();
 
     static_cast<Akregator::Part*>(m_part)->saveProperties(config);
-    config->writeEntry("docked", isHidden());
+    config.writeEntry("docked", isHidden());
 
     //delete m_part;
 }
 
-void MainWindow::readProperties(KConfig* config)
+void MainWindow::readProperties(const KConfigGroup & config)
 {
     if (!m_part)
         loadPart();
     static_cast<Akregator::Part*>(m_part)->readProperties(config);
-    
-    if (Settings::showTrayIcon() && config->readBoolEntry("docked", false)) 
+
+    if (Settings::showTrayIcon() && config.readEntry("docked", false))
         hide();
     else
         show();
@@ -188,21 +188,22 @@ void MainWindow::readProperties(KConfig* config)
 
 void MainWindow::optionsConfigureKeys()
 {
-    KKeyDialog dlg( true, this );
+    KShortcutsDialog dlg( KShortcutsEditor::AllActions,
+      KShortcutsEditor::LetterShortcutsDisallowed, this );
 
-    dlg.insert(actionCollection());
+    dlg.addCollection(actionCollection());
     if (m_part)
-        dlg.insert(m_part->actionCollection());
+        dlg.addCollection(m_part->actionCollection());
 
     dlg.configure();
 }
 
 void MainWindow::optionsConfigureToolbars()
 {
-    saveMainWindowSettings(KGlobal::config(), autoSaveGroup());
+    saveMainWindowSettings(KGlobal::config().data()->group( autoSaveGroup()) );
 
     // use the standard toolbar editor
-    KEditToolbar dlg(factory());
+    KEditToolBar dlg(factory());
     connect(&dlg, SIGNAL(newToolbarConfig()),
             this, SLOT(applyNewToolbarConfig()));
     dlg.exec();
@@ -212,7 +213,7 @@ void MainWindow::optionsConfigureToolbars()
 
 void MainWindow::applyNewToolbarConfig()
 {
-    applyMainWindowSettings(KGlobal::config(), autoSaveGroup());
+    applyMainWindowSettings(KGlobal::config()->group( autoSaveGroup()) );
 }
 
 
@@ -221,27 +222,16 @@ KParts::BrowserExtension *MainWindow::browserExtension(KParts::ReadOnlyPart *p)
     return KParts::BrowserExtension::childObject( p );
 }
 
-
-// from konqmainwindow
-void MainWindow::connectActionCollection( KActionCollection *coll )
-{
-    if (!coll) return;
-    connect( coll, SIGNAL( actionStatusText( const QString & ) ),
-              m_statusLabel, SLOT( setText( const QString & ) ) );
-    connect( coll, SIGNAL( clearStatusText() ),
-             this, SLOT( slotClearStatusText() ) );
-}
-
 bool MainWindow::queryExit()
 {
-    kdDebug() << "MainWindow::queryExit()" << endl;
+    kDebug() <<"MainWindow::queryExit()";
     if ( !kapp->sessionSaving() )
     {
         delete m_part; // delete that here instead of dtor to ensure nested khtmlparts are deleted before singleton objects like KHTMLPageCache
         m_part = 0;
     }
     else
-        kdDebug("MainWindow::queryExit(): saving session");
+        kDebug("MainWindow::queryExit(): saving session");
 
     return KMainWindow::queryExit();
 }
@@ -255,17 +245,22 @@ void MainWindow::slotQuit()
 
 bool MainWindow::queryClose()
 {
-    if (kapp->sessionSaving() || TrayIcon::getInstance() == 0 || TrayIcon::getInstance()->isHidden() )
+    if (kapp->sessionSaving() || TrayIcon::getInstance() == 0 || !TrayIcon::getInstance()->isVisible() )
     {
         return true;
     }
     else
     {
+#ifdef __GNUC__
+#warning takeScreenShot has to be reimplemented or removed
+#endif
+#if 0
         QPixmap shot = TrayIcon::getInstance()->takeScreenshot();
 
         // Associate source to image and show the dialog:
-        QMimeSourceFactory::defaultFactory()->setPixmap("systray_shot", shot);
-        KMessageBox::information(this, i18n( "<qt><p>Closing the main window will keep Akregator running in the system tray. Use 'Quit' from the 'File' menu to quit the application.</p><p><center><img source=\"systray_shot\"></center></p></qt>" ), i18n( "Docking in System Tray" ), "hideOnCloseInfo");
+        Q3MimeSourceFactory::defaultFactory()->setPixmap("systray_shot", shot);
+        KMessageBox::information(this, i18n( "<qt><p>Closing the main window will keep Akregator running in the system tray. Use 'Quit' from the 'File' menu to quit the application.</p><p><center><img source=\"systray_shot\" /></center></p></qt>" ), i18n( "Docking in System Tray" ), "hideOnCloseInfo");
+#endif
         hide();
         return false;
     }
