@@ -22,10 +22,9 @@
     without including the source code for Qt in the source distribution.
 */
 
-#ifndef TABWIDGET_H
-#define TABWIDGET_H
+#ifndef AKREGATOR_TABWIDGET_H
+#define AKREGATOR_TABWIDGET_H
 
-#include <qptrlist.h>
 #include <ktabwidget.h>
 
 class QString;
@@ -34,44 +33,40 @@ namespace Akregator
 {
 
 class Frame;
+class OpenUrlRequest;
 
 class TabWidget : public KTabWidget
 {
     Q_OBJECT
 
     public:
-        TabWidget(QWidget * parent = 0, const char *name = 0);
-        virtual ~TabWidget();
-
-        void addFrame(Frame *f);
-        Frame* currentFrame();
-        void removeFrame(Frame *f);
-        QPtrList<Frame> frames() const;
-
+        explicit TabWidget(QWidget * parent = 0);
+        ~TabWidget();
+       
     public slots:
 
-        void slotSetTitle(Frame* frame, const QString& title);
+        void slotSetTitle(Akregator::Frame* frame, const QString& title);
         void slotSettingsChanged();
         void slotNextTab();
         void slotPreviousTab();
         void slotRemoveCurrentFrame();
-
-    signals:
-        
-        void currentFrameChanged(Frame *);
-        
-    protected slots:
-
-        virtual void initiateDrag(int tab);
-        
-    private: // methods
-        
-        uint tabBarWidthForMaxChars( uint maxLength );
-        void setTitle( const QString &title , QWidget* sender);
+        void slotAddFrame(Akregator::Frame* f);
+        void slotRemoveFrame(int frameId);
+        void slotSelectFrame(int frameId);
 
 
-    private slots:
+    Q_SIGNALS:
         
+        void signalCurrentFrameChanged(int);
+        void signalRemoveFrameRequest(int);
+        void signalOpenUrlRequest(Akregator::OpenUrlRequest&);
+
+    private:
+        /*reimpl*/ void tabInserted( int );        
+        /*reimpl*/ void tabRemoved( int );
+
+    private Q_SLOTS:
+        void initiateDrag(int tab);
         void slotDetachTab();
         void slotCopyLinkAddress();
         void slotCloseTab();
@@ -81,10 +76,10 @@ class TabWidget : public KTabWidget
         
    private:
 
-       class TabWidgetPrivate;
-       TabWidgetPrivate* d;
+       class Private;
+       Private* const d;
 };
 
-}
+} // namespace Akregator
 
-#endif
+#endif // AKREGATOR_TABWIDGET_H
