@@ -22,43 +22,46 @@
     without including the source code for Qt in the source distribution.
 */
 
-#ifndef AKREGATORTRAYICON_H
-#define AKREGATORTRAYICON_H
+#ifndef AKREGATOR_TRAYICON_H
+#define AKREGATOR_TRAYICON_H
 
-#include <ksystemtray.h>
+#include "akregator_export.h"
+#include <ksystemtrayicon.h>
 
-#include <qimage.h>
-#include <qpixmap.h>
+#include <QImage>
+#include <QPixmap>
 
-namespace Akregator
+
+namespace Akregator {
+
+class AKREGATOR_EXPORT TrayIcon : public KSystemTrayIcon
 {
-    class TrayIcon : public KSystemTray
-    {
-        Q_OBJECT
-        public:
-            static TrayIcon* getInstance();
-            static void setInstance(TrayIcon* trayIcon);
-            
-            TrayIcon(QWidget *parent = 0, const char *name = 0);
-            ~TrayIcon();
-            
-            QPixmap takeScreenshot() const;
-            virtual void mousePressEvent(QMouseEvent *);
-        public slots:
-            void settingsChanged();
-            void slotSetUnread(int unread);
-            void viewButtonClicked();
-        
-        signals:
-            void showPart();
+    Q_OBJECT
+    public:
+        static TrayIcon* getInstance();
+        static void setInstance(TrayIcon* trayIcon);
 
-        private:
-            static TrayIcon* m_instance;
-            
-            QPixmap m_defaultIcon;
-            QImage m_lightIconImage;
-            int m_unread;
-    };
-}
+        explicit TrayIcon(QWidget *parent = 0);
+        ~TrayIcon();
 
-#endif
+        //QPixmap takeScreenshot() const;
+
+    public slots:
+        void settingsChanged();
+        void slotSetUnread(int unread);
+        void viewButtonClicked();
+        void slotActivated(QSystemTrayIcon::ActivationReason reason);
+
+    signals:
+        void showPart();
+
+    private:
+        static TrayIcon* m_instance;
+
+        QPixmap m_defaultIcon;
+        int m_unread;
+};
+
+} // namespace Akregator
+
+#endif // AKREGATOR_TRAYICON_H
