@@ -1,7 +1,7 @@
 /*
     This file is part of Akregator.
 
-    Copyright (C) 2004 Frank Osterfeld <frank.osterfeld at kdemail.net>
+    Copyright (C) 2004 Frank Osterfeld <osterfeld@kde.org>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -25,17 +25,19 @@
 #include "folder.h"
 #include "treenode.h"
 
-#include <qstring.h>
-#include <qvaluelist.h>
+#include <QString>
+#include <QList>
 
 #include <kdebug.h>
 
+#include <cassert>
+
 namespace Akregator {
 
 class TreeNode::TreeNodePrivate
 {
     public:
-    
+    TreeNodePrivate();
     bool doNotify;
     bool nodeChangeOccurred;
     bool articleChangeOccurred;
@@ -45,23 +47,27 @@ class TreeNode::TreeNodePrivate
     bool signalDestroyedEmitted;
 };
 
+TreeNode::TreeNodePrivate::TreeNodePrivate() : doNotify( true ), 
+                                               nodeChangeOccurred( false ), 
+                                               articleChangeOccurred( false ), 
+                                               title(), 
+                                               parent( 0 ),
+                                               id ( 0 ),
+                                               signalDestroyedEmitted( false )
+{
+}
+
 TreeNode::TreeNode()
-    : QObject(0, 0), d(new TreeNodePrivate)
+    : QObject(0), d(new TreeNodePrivate)
 {
-    d->doNotify = true;
-    d->nodeChangeOccurred = false;
-    d->articleChangeOccurred = false;
-    d->title = "";
-    d->parent = 0;
-    d->id = 0;
-    d->signalDestroyedEmitted = false;
-    
 }
 
 void TreeNode::emitSignalDestroyed()
 {
     if (!d->signalDestroyedEmitted)
     {
+        if ( parent() )
+            parent()->removeChild( this );
         emit signalDestroyed(this);
         d->signalDestroyedEmitted = true;
     } 
@@ -69,12 +75,12 @@ void TreeNode::emitSignalDestroyed()
 
 TreeNode::~TreeNode()
 {
-
+    assert( d->signalDestroyedEmitted || !"TreeNode subclasses must call emitSignalDestroyed in their destructor" );
     delete d;
     d = 0;
 }
 
-const QString& TreeNode::title() const
+QString TreeNode::title() const
 {
     return d->title;
 }
@@ -89,52 +95,93 @@ void TreeNode::setTitle(const QString& title)
     }
 }
 
-TreeNode* TreeNode::nextSibling() const
+TreeNode* TreeNode::nextSibling()
 {
     if (!d->parent)
         return 0;
-    QValueList<TreeNode*> children = d->parent->children();
-    TreeNode* me = (TreeNode*)this;
-        
-    int idx = children.findIndex(me);
+    const QList<TreeNode*> children = parent()->children();
+    const int idx = children.indexOf( this );
     
-    return idx+1 < children.size() ? *(children.at(idx+1)) : 0L;
+    return (idx+1 < children.size()) ? children.at(idx+1) : 0L;
 }
 
-TreeNode* TreeNode::prevSibling() const
+const TreeNode* TreeNode::nextSibling() const
 {
     if (!d->parent)
         return 0;
-    QValueList<TreeNode*> children = d->parent->children();
-    TreeNode* me = (TreeNode*)this;
+    const QList<const TreeNode*> children = parent()->children();
+    const int idx = children.indexOf( this );
     
-    int idx = children.findIndex(me);
-    return idx > 0 ? *(d->parent->children().at(idx-1)) : 0L;
+    return (idx+1 < children.size()) ? children.at(idx+1) : 0L;
 }
 
-Folder* TreeNode::parent() const
+TreeNode* TreeNode::prevSibling()
+{
+    if (!d->parent)
+        return 0;
+    const QList<TreeNode*> children = parent()->children();
+    
+    const int idx = children.indexOf( this );
+    return (idx > 0) ? children.at(idx-1) : 0L;
+}
+
+const TreeNode* TreeNode::prevSibling() const
+{
+    if (!d->parent)
+        return 0;
+    const QList<const TreeNode*> children = parent()->children();
+    const int idx = children.indexOf( this );
+    return (idx > 0) ? children.at(idx-1) : 0L;
+}
+
+const Folder* TreeNode::parent() const
 {
     return d->parent;
 }
 
+Folder* TreeNode::parent()
+{
+    return d->parent;
+}
+
+QList<const TreeNode*> TreeNode::children() const
+{
+    return QList<const TreeNode*>();
+}
+
+QList<TreeNode*> TreeNode::children()
+{
+    return QList<TreeNode*>();
+}
+
+const TreeNode* TreeNode::childAt( int pos ) const
+{
+    return 0;
+}
+
+TreeNode* TreeNode::childAt( int pos )
+{
+    return 0;
+}
+
 void TreeNode::setParent(Folder* parent)
 {
     d->parent = parent;
 }
 
-void TreeNode::setNotificationMode(bool doNotify, bool notifyOccurredChanges)
+void TreeNode::setNotificationMode( bool doNotify )
 {
-    if (doNotify && !d->doNotify) // turned on
+    if ( doNotify && !d->doNotify ) // turned on
     {
         d->doNotify = true;
-        if (d->nodeChangeOccurred && notifyOccurredChanges)
-            emit signalChanged(this);
-        if (d->articleChangeOccurred && notifyOccurredChanges)
+        if ( d->nodeChangeOccurred )
+            emit signalChanged( this );
+        if ( d->articleChangeOccurred )
             doArticleNotification();
         d->nodeChangeOccurred = false;
         d->articleChangeOccurred = false;
     }
-    if (!doNotify && d->doNotify) //turned off
+    else if ( !doNotify && d->doNotify ) //turned off
     {
         d->nodeChangeOccurred = false;
         d->articleChangeOccurred = false;
