@@ -21,25 +21,23 @@
     without including the source code for Qt in the source distribution.
 */
 
-#include <qcheckbox.h>
-#include <qhbox.h>
-#include <qlabel.h>
-#include <qlayout.h>
-#include <qlistbox.h>
-#include <qpushbutton.h>
-#include <qtabwidget.h>
-#include <qtextedit.h>
-#include <qtoolbutton.h>
-#include <qtooltip.h>
+#include <QCheckBox>
+
+#include <QLabel>
+#include <QPushButton>
+#include <QTabWidget>
+#include <QTextEdit>
+
+#include <QVBoxLayout>
+#include <QGridLayout>
 
 #include <kabc/resource.h>
+#include <kabc/resourceabc.h>
 #include <kabc/stdaddressbook.h>
-#include <kaccelmanager.h>
-#include <kapplication.h>
+#include <kacceleratormanager.h>
 #include <kconfig.h>
 #include <kcombobox.h>
 #include <kdebug.h>
-#include <kdialogbase.h>
 #include <kglobal.h>
 #include <kiconloader.h>
 #include <klineedit.h>
@@ -53,7 +51,7 @@
 #include <libkdepim/categoryeditdialog.h>
 #include <libkdepim/categoryselectdialog.h>
 #include <libkdepim/kdateedit.h>
-#include <libkdepim/resourceabc.h>
+#include <kvbox.h>
 
 #include "addresseditwidget.h"
 #include "advancedcustomfields.h"
@@ -67,11 +65,11 @@
 
 #include "addresseeeditorwidget.h"
 
-AddresseeEditorWidget::AddresseeEditorWidget( QWidget *parent, const char *name )
-  : AddresseeEditorBase( parent, name ),
+AddresseeEditorWidget::AddresseeEditorWidget( QWidget *parent )
+  : AddresseeEditorBase( parent ),
     mBlockSignals( false ), mReadOnly( false )
 {
-  kdDebug(5720) << "AddresseeEditorWidget()" << endl;
+  kDebug(5720) <<"AddresseeEditorWidget()";
 
   initGUI();
   mCategorySelectDialog = 0;
@@ -85,7 +83,7 @@ AddresseeEditorWidget::AddresseeEditorWidget( QWidget *parent, const char *name
 
 AddresseeEditorWidget::~AddresseeEditorWidget()
 {
-  kdDebug(5720) << "~AddresseeEditorWidget()" << endl;
+  kDebug(5720) <<"~AddresseeEditorWidget()";
 }
 
 void AddresseeEditorWidget::setAddressee( const KABC::Addressee &addr )
@@ -99,9 +97,9 @@ void AddresseeEditorWidget::setAddressee( const KABC::Addressee &addr )
     if ( res->readOnly() ) {
       readOnly = true;
 
-    //Kolab resources have finer access control than planned in the overall design.
-    } else if ( res->inherits( "KPIM::ResourceABC" ) ) {
-      KPIM::ResourceABC *resAbc = static_cast<KPIM::ResourceABC *>( res );
+    //HACK: some resources have finer access control than "generic" resources
+    } else if ( res->inherits( "KPIM::ResourceAbc" ) ) {
+      KABC::ResourceABC *resAbc = static_cast<KABC::ResourceABC *>( res );
 
       QString subresource = resAbc->uidToResourceMap()[ addr.uid() ];
       if ( !subresource.isEmpty() )
@@ -126,6 +124,8 @@ void AddresseeEditorWidget::textChanged( const QString& )
 void AddresseeEditorWidget::initGUI()
 {
   QVBoxLayout *layout = new QVBoxLayout( this );
+  layout->setMargin( 0 );
+  layout->setSpacing( KDialog::spacingHint() );
 
   mTabWidget = new QTabWidget( this );
   layout->addWidget( mTabWidget );
@@ -144,9 +144,9 @@ void AddresseeEditorWidget::setupTab1()
   // This is the General tab
   QWidget *tab1 = new QWidget( mTabWidget );
 
-  QGridLayout *layout = new QGridLayout( tab1, 11, 7 );
-  layout->setMargin( KDialogBase::marginHint() );
-  layout->setSpacing( KDialogBase::spacingHint() );
+  QGridLayout *layout = new QGridLayout( tab1 );
+  layout->setMargin( KDialog::marginHint() );
+  layout->setSpacing( KDialog::spacingHint() );
 
   QLabel *label;
   KSeparator* bar;
@@ -157,14 +157,14 @@ void AddresseeEditorWidget::setupTab1()
 
   // Person icon
   label = new QLabel( tab1 );
-  label->setPixmap( KGlobal::iconLoader()->loadIcon( "personal", KIcon::Desktop,
-                                                      KIcon::SizeMedium ) );
-  layout->addMultiCellWidget( label, 0, 1, 0, 0 );
+  label->setPixmap( KIconLoader::global()->loadIcon( "user-identity", KIconLoader::Desktop,
+                                                      KIconLoader::SizeMedium ) );
+  layout->addWidget( label, 0, 0, 2, 1);
 
   // First name
   button = new QPushButton( i18n( "Edit Name..." ), tab1 );
-  QToolTip::add( button, i18n( "Edit the contact's name" ) );
-  mNameEdit = new KLineEdit( tab1, "mNameEdit" );
+  button->setToolTip( i18n( "Edit the contact's name" ) );
+  mNameEdit = new KLineEdit( tab1 );
   connect( mNameEdit, SIGNAL( textChanged( const QString& ) ),
            SLOT( nameTextChanged( const QString& ) ) );
   connect( button, SIGNAL( clicked() ), SLOT( nameButtonClicked() ) );
@@ -181,7 +181,7 @@ void AddresseeEditorWidget::setupTab1()
   layout->addWidget( button, 0, 1 );
   layout->addWidget( mNameEdit, 0, 2 );
   layout->addWidget( mNameLabel, 0, 2 );
-  label = new QLabel( i18n( "<roleLabel>:", "%1:" ).arg( KABC::Addressee::roleLabel() ), tab1 );
+  label = new QLabel( i18nc( "<roleLabel>:", "%1:", KABC::Addressee::roleLabel() ), tab1 );
   mRoleEdit = new KLineEdit( tab1 );
   connect( mRoleEdit, SIGNAL( textChanged( const QString& ) ),
            SLOT( textChanged( const QString& ) ) );
@@ -190,7 +190,7 @@ void AddresseeEditorWidget::setupTab1()
   layout->addWidget( mRoleEdit, 1, 2 );
 
   // Organization
-  label = new QLabel( i18n( "<organizationLabel>:", "%1:" ).arg( KABC::Addressee::organizationLabel() ), tab1 );
+  label = new QLabel( i18nc( "<organizationLabel>:", "%1:", KABC::Addressee::organizationLabel() ), tab1 );
   mOrgEdit = new KLineEdit( tab1 );
   label->setBuddy( mOrgEdit );
   connect( mOrgEdit, SIGNAL( textChanged( const QString& ) ),
@@ -206,87 +206,87 @@ void AddresseeEditorWidget::setupTab1()
 
   // Left hand separator. This separator doesn't go all the way
   // across so the dialog still flows from top to bottom
-  bar = new KSeparator( KSeparator::HLine, tab1 );
-  layout->addMultiCellWidget( bar, 4, 4, 0, 2 );
+  bar = new KSeparator( Qt::Horizontal, tab1 );
+  layout->addWidget( bar, 4, 0, 1, 3 );
 
   //////////////////////////////////////
   // Phone numbers (upper right)
   label = new QLabel( tab1 );
-  label->setPixmap( KGlobal::iconLoader()->loadIcon( "kaddressbook",
-                    KIcon::Desktop, KIcon::SizeMedium ) );
-  layout->addMultiCellWidget( label, 0, 1, 3, 3 );
+  label->setPixmap( KIconLoader::global()->loadIcon( "x-office-contact",
+                    KIconLoader::Desktop, KIconLoader::SizeMedium ) );
+  layout->addWidget( label, 0, 3, 2, 1 );
 
   mPhoneEditWidget = new PhoneEditWidget( tab1 );
   connect( mPhoneEditWidget, SIGNAL( modified() ), SLOT( emitModified() ) );
-  layout->addMultiCellWidget( mPhoneEditWidget, 0, 3, 4, 6 );
+  layout->addWidget( mPhoneEditWidget, 0, 4, 4, 3 );
 
-  bar = new KSeparator( KSeparator::HLine, tab1 );
-  layout->addMultiCellWidget( bar, 4, 4, 3, 6 );
+  bar = new KSeparator( Qt::Horizontal, tab1 );
+  layout->addWidget( bar, 4, 3, 1, 4 );
 
   //////////////////////////////////////
   // Addresses (lower left)
   label = new QLabel( tab1 );
-  label->setPixmap( KGlobal::iconLoader()->loadIcon( "kfm_home", KIcon::Desktop,
-                                                     KIcon::SizeMedium ) );
-  layout->addMultiCellWidget( label, 5, 6, 0, 0 );
+  label->setPixmap( KIconLoader::global()->loadIcon( "go-home", KIconLoader::Desktop,
+                                                     KIconLoader::SizeMedium ) );
+  layout->addWidget( label, 5, 0, 2, 1);
 
   mAddressEditWidget = new AddressEditWidget( tab1 );
   connect( mAddressEditWidget, SIGNAL( modified() ), SLOT( emitModified() ) );
-  layout->addMultiCellWidget( mAddressEditWidget, 5, 10, 1, 2 );
+  layout->addWidget( mAddressEditWidget, 5, 1, 6, 2 );
 
   //////////////////////////////////////
   // Email / Web (lower right)
   label = new QLabel( tab1 );
-  label->setPixmap( KGlobal::iconLoader()->loadIcon( "email", KIcon::Desktop,
-                                                     KIcon::SizeMedium ) );
-  layout->addMultiCellWidget( label, 5, 6, 3, 3 );
+  label->setPixmap( KIconLoader::global()->loadIcon( "mail-message", KIconLoader::Desktop,
+                                                     KIconLoader::SizeMedium ) );
+  layout->addWidget( label, 5, 3, 2, 1);
 
   mEmailWidget = new EmailEditWidget( tab1 );
   connect( mEmailWidget, SIGNAL( modified() ), SLOT( emitModified() ) );
-  layout->addMultiCellWidget( mEmailWidget, 5, 6, 4, 6 );
+  layout->addWidget( mEmailWidget, 5, 4, 2, 3 );
 
   // add the separator
-  bar = new KSeparator( KSeparator::HLine, tab1 );
-  layout->addMultiCellWidget( bar, 7, 7, 3, 6 );
-
-  QHBoxLayout *homePageLayout = new QHBoxLayout( 0, 11, 7 );
+  bar = new KSeparator( Qt::Horizontal, tab1 );
+  layout->addWidget( bar, 7, 3, 1, 4 );
 
   label = new QLabel( tab1 );
-  label->setPixmap( KGlobal::iconLoader()->loadIcon( "homepage", KIcon::Desktop,
-                                                     KIcon::SizeMedium ) );
-  homePageLayout->addWidget( label );
+  label->setPixmap( KIconLoader::global()->loadIcon( "internet-web-browser", KIconLoader::Desktop,
+                                                     KIconLoader::SizeMedium ) );
+  layout->addWidget( label, 9, 3, 1, 2, Qt::AlignTop );
+
+  QGridLayout *gridLayout = new QGridLayout;
+  gridLayout->setSpacing( 7 );
+  gridLayout->setMargin( 11 );
+
+  label = new QLabel( i18nc( "<urlLabel>:", "%1:", KABC::Addressee::urlLabel() ), tab1 );
 
-  label = new QLabel( i18n( "<urlLabel>:", "%1:" ).arg( KABC::Addressee::urlLabel() ), tab1 );
   mURLEdit = new KLineEdit( tab1 );
   connect( mURLEdit, SIGNAL( textChanged( const QString& ) ),
            SLOT( textChanged( const QString& ) ) );
   label->setBuddy( mURLEdit );
-  homePageLayout->addWidget( label );
-  homePageLayout->addWidget( mURLEdit );
-  layout->addMultiCellLayout( homePageLayout, 8, 8, 3, 6 );
+  gridLayout->addWidget( label, 0, 0 );
+  gridLayout->addWidget( mURLEdit, 0, 1 );
 
-  QHBoxLayout *blogLayout = new QHBoxLayout( 0, 11, 7 );
-  label = new QLabel( i18n("Blog feed:"), tab1 );
-  blogLayout->addWidget( label );
+  label = new QLabel( i18n( "Blog feed:" ), tab1 );
   mBlogEdit = new KLineEdit( tab1 );
-  blogLayout->addWidget( mBlogEdit );
+  gridLayout->addWidget( label, 1, 0 );
+  gridLayout->addWidget( mBlogEdit, 1, 1 );
   connect( mBlogEdit, SIGNAL( textChanged( const QString & ) ),
            SLOT( textChanged( const QString & ) ) );
   label->setBuddy( mBlogEdit );
-  layout->addMultiCellLayout( blogLayout, 9, 9, 4, 6 );
 
-  mIMWidget = new IMEditWidget( tab1, mAddressee );
+  mIMWidget = new IMEditWidget( tab1, mAddressee, gridLayout );
   connect( mIMWidget, SIGNAL( modified() ), SLOT( emitModified() ) );
-  layout->addMultiCellWidget( mIMWidget, 10, 10, 4, 6 );
+  layout->addLayout( gridLayout, 8, 4, 3, 3 );
 
-  layout->addColSpacing( 6, 50 );
+  layout->addItem( new QSpacerItem( 50, 0 ), 0, 6 );
 
-  bar = new KSeparator( KSeparator::HLine, tab1 );
-  layout->addMultiCellWidget( bar, 11, 11, 0, 6 );
+  bar = new KSeparator( Qt::Horizontal, tab1 );
+  layout->addWidget( bar, 11, 0, 1, 7 );
 
   ///////////////////////////////////////
-  QHBox *categoryBox = new QHBox( tab1 );
-  categoryBox->setSpacing( KDialogBase::spacingHint() );
+  KHBox *categoryBox = new KHBox( tab1 );
+  categoryBox->setSpacing( KDialog::spacingHint() );
 
   // Categories
   mCategoryButton = new QPushButton( i18n( "Select Categories..." ), categoryBox );
@@ -300,7 +300,7 @@ void AddresseeEditorWidget::setupTab1()
   mSecrecyWidget = new SecrecyWidget( categoryBox );
   connect( mSecrecyWidget, SIGNAL( changed() ), SLOT( emitModified() ) );
 
-  layout->addMultiCellWidget( categoryBox, 12, 12, 0, 6 );
+  layout->addWidget( categoryBox, 12, 0, 1, 7 );
 
   // Build the layout and add to the tab widget
   layout->activate(); // required
@@ -313,9 +313,9 @@ void AddresseeEditorWidget::setupTab2()
   // This is the Details tab
   QWidget *tab2 = new QWidget( mTabWidget );
 
-  QGridLayout *layout = new QGridLayout( tab2, 6, 6 );
-  layout->setMargin( KDialogBase::marginHint() );
-  layout->setSpacing( KDialogBase::spacingHint() );
+  QGridLayout *layout = new QGridLayout( tab2 );
+  layout->setMargin( KDialog::marginHint() );
+  layout->setSpacing( KDialog::spacingHint() );
 
   QLabel *label;
   KSeparator* bar;
@@ -325,9 +325,9 @@ void AddresseeEditorWidget::setupTab2()
 
   // Department
   label = new QLabel( tab2 );
-  label->setPixmap( KGlobal::iconLoader()->loadIcon( "folder", KIcon::Desktop,
-                                                     KIcon::SizeMedium ) );
-  layout->addMultiCellWidget( label, 0, 1, 0, 0 );
+  label->setPixmap( KIconLoader::global()->loadIcon( "folder", KIconLoader::Desktop,
+                                                     KIconLoader::SizeMedium ) );
+  layout->addWidget( label, 0, 0, 2, 1 );
 
   label = new QLabel( i18n( "Department:" ), tab2 );
   layout->addWidget( label, 0, 1 );
@@ -359,7 +359,7 @@ void AddresseeEditorWidget::setupTab2()
   connect( mManagerEdit, SIGNAL( textChanged( const QString& ) ),
            SLOT( textChanged( const QString& ) ) );
   label->setBuddy( mManagerEdit );
-  layout->addMultiCellWidget( mManagerEdit, 0, 0, 4, 5 );
+  layout->addWidget( mManagerEdit, 0, 4, 1, 2 );
 
   label = new QLabel( i18n( "Assistant's name:" ), tab2 );
   layout->addWidget( label, 1, 3 );
@@ -367,26 +367,26 @@ void AddresseeEditorWidget::setupTab2()
   connect( mAssistantEdit, SIGNAL( textChanged( const QString& ) ),
            SLOT( textChanged( const QString& ) ) );
   label->setBuddy( mAssistantEdit );
-  layout->addMultiCellWidget( mAssistantEdit, 1, 1, 4, 5 );
+  layout->addWidget( mAssistantEdit, 1, 4, 1, 2 );
 
-  label = new QLabel( i18n( "<titleLabel>:", "%1:" ).arg( KABC::Addressee::titleLabel() ), tab2 );
+  label = new QLabel( i18nc( "<titleLabel>:", "%1:", KABC::Addressee::titleLabel() ), tab2 );
   layout->addWidget( label, 2, 3 );
   mTitleEdit = new KLineEdit( tab2 );
   connect( mTitleEdit, SIGNAL( textChanged( const QString& ) ),
            SLOT( textChanged( const QString& ) ) );
   label->setBuddy( mTitleEdit );
-  layout->addMultiCellWidget( mTitleEdit, 2, 2, 4, 5 );
+  layout->addWidget( mTitleEdit, 2, 4, 1, 2 );
 
-  bar = new KSeparator( KSeparator::HLine, tab2 );
-  layout->addMultiCellWidget( bar, 3, 3, 0, 5 );
+  bar = new KSeparator( Qt::Horizontal, tab2 );
+  layout->addWidget( bar, 3, 0, 1, 6 );
 
   /////////////////////////////////////////////////
   // Personal info
 
   label = new QLabel( tab2 );
-  label->setPixmap( KGlobal::iconLoader()->loadIcon( "personal", KIcon::Desktop,
-                                                     KIcon::SizeMedium ) );
-  layout->addMultiCellWidget( label, 4, 5, 0, 0 );
+  label->setPixmap( KIconLoader::global()->loadIcon( "user-identity", KIconLoader::Desktop,
+                                                     KIconLoader::SizeMedium ) );
+  layout->addWidget( label, 4, 0, 2, 1);
 
   label = new QLabel( i18n( "Nickname:" ), tab2 );
   layout->addWidget( label, 4, 1 );
@@ -396,7 +396,7 @@ void AddresseeEditorWidget::setupTab2()
   label->setBuddy( mNicknameEdit );
   layout->addWidget( mNicknameEdit, 4, 2 );
 
-  label = new QLabel( i18n( "Partner's name:" ), tab2 );
+  label = new QLabel( i18nc( "Wife/Husband/...", "Partner's name:" ), tab2 );
   layout->addWidget( label, 5, 1 );
   mSpouseEdit = new KPIM::AddresseeLineEdit( tab2 );
   connect( mSpouseEdit, SIGNAL( textChanged( const QString& ) ),
@@ -406,7 +406,7 @@ void AddresseeEditorWidget::setupTab2()
 
   label = new QLabel( i18n( "Birthdate:" ), tab2 );
   layout->addWidget( label, 4, 3 );
-  mBirthdayPicker = new KDateEdit( tab2 );
+  mBirthdayPicker = new KPIM::KDateEdit( tab2 );
   connect( mBirthdayPicker, SIGNAL( dateChanged( const QDate& ) ),
            SLOT( dateChanged( const QDate& ) ) );
   connect( mBirthdayPicker, SIGNAL( textChanged( const QString& ) ),
@@ -416,7 +416,7 @@ void AddresseeEditorWidget::setupTab2()
 
   label = new QLabel( i18n( "Anniversary:" ), tab2 );
   layout->addWidget( label, 5, 3 );
-  mAnniversaryPicker = new KDateEdit( tab2 );
+  mAnniversaryPicker = new KPIM::KDateEdit( tab2 );
   connect( mAnniversaryPicker, SIGNAL( dateChanged( const QDate& ) ),
            SLOT( dateChanged( const QDate& ) ) );
   connect( mAnniversaryPicker, SIGNAL( textChanged( const QString& ) ),
@@ -424,8 +424,8 @@ void AddresseeEditorWidget::setupTab2()
   label->setBuddy( mAnniversaryPicker );
   layout->addWidget( mAnniversaryPicker, 5, 4 );
 
-  bar = new KSeparator( KSeparator::HLine, tab2 );
-  layout->addMultiCellWidget( bar, 6, 6, 0, 5 );
+  bar = new KSeparator( Qt::Horizontal, tab2 );
+  layout->addWidget( bar, 6, 0, 1, 6 );
 
    //////////////////////////////////////
   // Notes
@@ -433,11 +433,11 @@ void AddresseeEditorWidget::setupTab2()
   label->setAlignment( Qt::AlignTop | Qt::AlignLeft );
   layout->addWidget( label, 7, 0 );
   mNoteEdit = new QTextEdit( tab2 );
-  mNoteEdit->setWordWrap( QTextEdit::WidgetWidth );
+  mNoteEdit->setWordWrapMode ( QTextOption::WrapAnywhere );
   mNoteEdit->setMinimumSize( mNoteEdit->sizeHint() );
   connect( mNoteEdit, SIGNAL( textChanged() ), SLOT( emitModified() ) );
   label->setBuddy( mNoteEdit );
-  layout->addMultiCellWidget( mNoteEdit, 7, 7, 1, 5 );
+  layout->addWidget( mNoteEdit, 7, 1, 1, 5 );
 
    // Build the layout and add to the tab widget
   layout->activate(); // required
@@ -475,18 +475,22 @@ void AddresseeEditorWidget::setupAdditionalTabs()
   }
 
   // query the layout update
-  QDictIterator<ContactEditorTabPage> it( mTabPages );
-  for ( ; it.current(); ++it )
-    it.current()->updateLayout();
+  QHashIterator<QString, ContactEditorTabPage*> it( mTabPages );
+  while ( it.hasNext() ) {
+    it.next();
+    it.value()->updateLayout();
+  }
 }
 
 void AddresseeEditorWidget::setupCustomFieldsTabs()
 {
   QStringList activePages = KABPrefs::instance()->advancedCustomFields();
 
-  const QStringList list = KGlobal::dirs()->findAllResources( "data", "kaddressbook/contacteditorpages/*.ui", true, true );
+  const QStringList list = KGlobal::dirs()->findAllResources( "data", "kaddressbook/contacteditorpages/*.ui",
+                                                              KStandardDirs::Recursive |
+                                                              KStandardDirs::NoDuplicates );
   for ( QStringList::ConstIterator it = list.begin(); it != list.end(); ++it ) {
-    if ( activePages.find( (*it).mid( (*it).findRev('/') + 1 ) ) == activePages.end() )
+    if ( !activePages.contains( (*it).mid( (*it).lastIndexOf('/') + 1 ) )  )
       continue;
 
     ContactEditorTabPage *page = new ContactEditorTabPage( mTabWidget );
@@ -506,7 +510,7 @@ void AddresseeEditorWidget::setupCustomFieldsTabs()
 
 void AddresseeEditorWidget::load()
 {
-  kdDebug(5720) << "AddresseeEditorWidget::load()" << endl;
+  kDebug(5720) <<"AddresseeEditorWidget::load()";
 
   // Block signals in case anything tries to emit modified
   // CS: This doesn't seem to work.
@@ -519,9 +523,9 @@ void AddresseeEditorWidget::load()
   mNameEdit->blockSignals( false );
 
   if ( mAddressee.formattedName().isEmpty() ) {
-    KConfig config( "kaddressbookrc" );
-    config.setGroup( "General" );
-    mFormattedNameType = config.readNumEntry( "FormattedNameType", 1 );
+    KConfig _config( "kaddressbookrc" );
+    KConfigGroup config(&_config, "General" );
+    mFormattedNameType = config.readEntry( "FormattedNameType", 1 );
     mAddressee.setFormattedName( NameEditDialog::formattedName( mAddressee, mFormattedNameType ) );
   } else {
     if ( mAddressee.formattedName() == NameEditDialog::formattedName( mAddressee, NameEditDialog::SimpleName ) )
@@ -542,15 +546,13 @@ void AddresseeEditorWidget::load()
 
   mRoleEdit->setText( mAddressee.role() );
   mOrgEdit->setText( mAddressee.organization() );
-#if KDE_IS_VERSION(3,5,8)
   mDepartmentEdit->setText( mAddressee.department() );
   // compatibility with older versions
   if ( mAddressee.department().isEmpty() )
-#endif
     mDepartmentEdit->setText( mAddressee.custom( "KADDRESSBOOK", "X-Department" ) );
-  mURLEdit->setURL( mAddressee.url() );
+  mURLEdit->setUrl( mAddressee.url() );
   mURLEdit->home( false );
-  mBlogEdit->setURL( mAddressee.custom( "KADDRESSBOOK", "BlogFeed" ) );
+  mBlogEdit->setUrl( mAddressee.custom( "KADDRESSBOOK", "BlogFeed" ) );
   mNoteEdit->setText( mAddressee.note() );
   mEmailWidget->setEmails( mAddressee.emails() );
   mPhoneEditWidget->setPhoneNumbers( mAddressee.phoneNumbers() );
@@ -574,9 +576,11 @@ void AddresseeEditorWidget::load()
   mProfessionEdit->setText( mAddressee.custom( "KADDRESSBOOK", "X-Profession" ) );
   mTitleEdit->setText( mAddressee.title() );
 
-  QDictIterator<ContactEditorTabPage> it( mTabPages );
-  for ( ; it.current(); ++it )
-    it.current()->loadContact( &mAddressee );
+  QHashIterator<QString, ContactEditorTabPage*> it( mTabPages );
+  while ( it.hasNext() ) {
+    it.next();
+    it.value()->loadContact( &mAddressee );
+  }
 
   blockSignals( block );
   mBlockSignals = false;
@@ -590,36 +594,21 @@ void AddresseeEditorWidget::save()
 
   mAddressee.setRole( mRoleEdit->text() );
   mAddressee.setOrganization( mOrgEdit->text() );
-#if KDE_IS_VERSION(3,5,8)
   mAddressee.setDepartment( mDepartmentEdit->text() );
-#else
-  if ( !mDepartmentEdit->text().isEmpty() )
-    mAddressee.insertCustom( "KADDRESSBOOK", "X-Department", mDepartmentEdit->text() );
-  else
-    mAddressee.removeCustom( "KADDRESSBOOK", "X-Department" );
-#endif
-
-  QString homepage = mURLEdit->text().stripWhiteSpace();
-  if ( homepage.isEmpty() )
-     mAddressee.setUrl( KURL() );
-  else {
-     if( !homepage.startsWith("http") )
-       homepage = "http://" + homepage;
-     mAddressee.setUrl( KURL( homepage ) );
-  }
+  mAddressee.setUrl( KUrl( mURLEdit->text().trimmed() ) );
   if ( !mBlogEdit->text().isEmpty() )
     mAddressee.insertCustom( "KADDRESSBOOK", "BlogFeed", mBlogEdit->text() );
   else
     mAddressee.removeCustom( "KADDRESSBOOK", "BlogFeed" );
 
-  mAddressee.setNote( mNoteEdit->text() );
+  mAddressee.setNote( mNoteEdit->toPlainText() );
   if ( mBirthdayPicker->date().isValid() )
     mAddressee.setBirthday( QDateTime( mBirthdayPicker->date() ) );
   else
     mAddressee.setBirthday( QDateTime() );
 
   mAddressee.setNickName( mNicknameEdit->text() );
-  mAddressee.setCategories( QStringList::split( ",", mCategoryEdit->text() ) );
+  mAddressee.setCategories( mCategoryEdit->text().split( ",", QString::SkipEmptyParts ) );
 
   mAddressee.setSecrecy( mSecrecyWidget->secrecy() );
 
@@ -687,9 +676,11 @@ void AddresseeEditorWidget::save()
         ++addressIter )
     mAddressee.insertAddress( *addressIter );
 
-  QDictIterator<ContactEditorTabPage> it( mTabPages );
-  for ( ; it.current(); ++it )
-    it.current()->storeContact( &mAddressee );
+  QHashIterator<QString, ContactEditorTabPage*> it( mTabPages );
+  while ( it.hasNext() ) {
+    it.next();
+    it.value()->storeContact( &mAddressee );
+  }
 
   mDirty = false;
 }
@@ -804,8 +795,9 @@ void AddresseeEditorWidget::selectCategories()
              this, SLOT( editCategories() ) );
   }
 
-  mCategorySelectDialog->setSelected( QStringList::split( ",", mCategoryEdit->text() ) );
-  mCategorySelectDialog->exec();
+  mCategorySelectDialog->setSelected( mCategoryEdit->text().split( ",", QString::SkipEmptyParts) );
+  mCategorySelectDialog->show();
+  mCategorySelectDialog->raise();
 }
 
 void AddresseeEditorWidget::categoriesSelected( const QStringList &list )
@@ -821,7 +813,8 @@ void AddresseeEditorWidget::editCategories()
              mCategorySelectDialog, SLOT( updateCategoryConfig() ) );
   }
 
-  mCategoryEditDialog->exec();
+  mCategoryEditDialog->show();
+  mCategoryEditDialog->raise();
 }
 
 void AddresseeEditorWidget::emitModified()
@@ -901,9 +894,11 @@ void AddresseeEditorWidget::setReadOnly( bool readOnly )
   mAnniversaryPicker->setEnabled( !readOnly );
   mNoteEdit->setReadOnly( mReadOnly );
 
-  QDictIterator<ContactEditorTabPage> it( mTabPages );
-  for ( ; it.current(); ++it )
-    it.current()->setReadOnly( readOnly );
+  QHashIterator<QString, ContactEditorTabPage*> it( mTabPages );
+  while ( it.hasNext() ) {
+    it.next();
+    it.value()->setReadOnly( readOnly );
+  }
 }
 
 #include "addresseeeditorwidget.moc"
