@@ -16,22 +16,17 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
     As a special exception, permission is given to link this program
     with any edition of Qt, and distribute the resulting executable,
     without including the source code for Qt in the source distribution.
 */
 
-#include <qcheckbox.h>
-#include <qcombobox.h>
-#include <qdatetimeedit.h>
-#include <qlayout.h>
-#include <qobjectlist.h>
-#include <qspinbox.h>
-#include <qregexp.h>
-#include <qtextedit.h>
-#include <qwidgetfactory.h>
+#include <QCheckBox>
+#include <QRegExp>
+//Added by qt3to4:
+#include <QVBoxLayout>
 
 #include <kdatepicker.h>
 #include <kdatetimewidget.h>
@@ -85,8 +80,8 @@ class KABCStorage : public KPIM::DesignerFields::Storage
 
 
 AdvancedCustomFields::AdvancedCustomFields( const QString &uiFile, KABC::AddressBook *ab,
-                                            QWidget *parent, const char *name )
-  : KAB::ContactEditorWidget( ab, parent, name )
+                                            QWidget *parent )
+  : KAB::ContactEditorWidget( ab, parent )
 {
   initGUI( uiFile );
 }
@@ -94,8 +89,8 @@ AdvancedCustomFields::AdvancedCustomFields( const QString &uiFile, KABC::Address
 void AdvancedCustomFields::loadContact( KABC::Addressee *addr )
 {
   QString ns;
-  if ( mFields->identifier().upper() == "KADDRESSBOOK" ||
-    QRegExp( "^Form\\d\\d?$" ).search( mFields->identifier() ) >= 0 ) {
+  if ( mFields->identifier().toUpper() == "KADDRESSBOOK" ||
+    QRegExp( "^Form\\d\\d?$" ).indexIn( mFields->identifier() ) >= 0 ) {
     ns = "KADDRESSBOOK";
   } else {
     ns = mFields->identifier();
@@ -108,8 +103,8 @@ void AdvancedCustomFields::loadContact( KABC::Addressee *addr )
 void AdvancedCustomFields::storeContact( KABC::Addressee *addr )
 {
   QString ns;
-  if ( mFields->identifier().upper() == "KADDRESSBOOK" ||
-    QRegExp( "^Form\\d\\d?$" ).search( mFields->identifier() ) >= 0 ) {
+  if ( mFields->identifier().toUpper() == "KADDRESSBOOK" ||
+    QRegExp( "^Form\\d\\d?$" ).indexIn( mFields->identifier() ) >= 0 ) {
     ns = "KADDRESSBOOK";
   } else {
     ns = mFields->identifier();
@@ -126,8 +121,9 @@ void AdvancedCustomFields::setReadOnly( bool readOnly )
 
 void AdvancedCustomFields::initGUI( const QString &uiFile )
 {
-  QVBoxLayout *layout = new QVBoxLayout( this, KDialog::marginHint(),
-                                         KDialog::spacingHint() );
+  QVBoxLayout *layout = new QVBoxLayout( this );
+  layout->setSpacing( KDialog::spacingHint() );
+  layout->setMargin( KDialog::marginHint() );
 
   mFields = new KPIM::DesignerFields( uiFile, this );
   layout->addWidget( mFields );
