@@ -12,9 +12,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
     As a special exception, permission is given to link this program
     with any edition of Qt, and distribute the resulting executable,
@@ -24,13 +24,11 @@
 #ifndef KABPREFS_H
 #define KABPREFS_H
 
-#include <qstringlist.h>
-#include <kdepimmacros.h>
+#include "kabcommon_export.h"
 #include "kabprefs_base.h"
+#include <QStringList>
 
-class KConfig;
-
-class KDE_EXPORT KABPrefs : public KABPrefsBase
+class KABCOMMON_EXPORT KABPrefs : public KABPrefsBase
 {
   public:
     virtual ~KABPrefs();
@@ -76,6 +74,7 @@ class KDE_EXPORT KABPrefs : public KABPrefsBase
 
   private:
     KABPrefs();
+    friend class KABPrefsHelper;
 
     static KABPrefs *mInstance;
 
