@@ -21,18 +21,19 @@
     without including the source code for Qt in the source distribution.
 */
 
-#include <kactionclasses.h>
+#include <kactioncollection.h>
 #include <kconfig.h>
 #include <kdebug.h>
 #include <klocale.h>
-#include <ktrader.h>
+#include <kservicetypetrader.h>
+#include <ktoggleaction.h>
+#include <kvbox.h>
 
-#include <qlayout.h>
-#include <qobjectlist.h>
-#include <qsignalmapper.h>
-#include <qsplitter.h>
-#include <qtimer.h>
-#include <qwidgetstack.h>
+#include <QHBoxLayout>
+#include <QSignalMapper>
+#include <QSplitter>
+#include <QStackedWidget>
+#include <QTimer>
 
 #include "addresseeeditorextension.h"
 #include "core.h"
@@ -44,22 +45,25 @@ ExtensionData::ExtensionData() : action( 0 ), widget( 0 ), weight( 0 ), isDetail
 {
 }
 
-ExtensionManager::ExtensionManager( QWidget* extensionBar, QWidgetStack* detailsStack, KAB::Core *core, QObject *parent,
-                                    const char *name )
-    : QObject( parent, name ), mExtensionBar( extensionBar ), mCore( core ),
-      mMapper( 0 ), mDetailsStack( detailsStack ), mActiveDetailsWidget( 0 )
+ExtensionManager::ExtensionManager( QWidget* extensionBar, QStackedWidget* detailsStack, KAB::Core *core, QObject *parent )
+    : QObject( parent ), mExtensionBar( extensionBar ), mCore( core ),
+    mMapper( 0 ), mDetailsStack( detailsStack ), mActiveDetailsWidget( 0 )
 {
+  Q_ASSERT( mCore );
   Q_ASSERT( mExtensionBar );
-  QVBoxLayout* layout = new QVBoxLayout( mExtensionBar );
+  Q_ASSERT( mDetailsStack );
+  QBoxLayout *layout = new QHBoxLayout( mExtensionBar );
+  layout->setMargin( 0 );
+  layout->setSpacing( 0 );
   mSplitter = new QSplitter( mExtensionBar );
-  mSplitter->setOrientation( QSplitter::Vertical );
+  mSplitter->setOrientation( Qt::Vertical );
+  mSplitter->setChildrenCollapsible( false );
   layout->addWidget( mSplitter );
 
   createExtensionWidgets();
 
-  mActionCollection = new KActionCollection( this, "ActionCollection" );
-
-  extensionBar->setShown( false );
+  mActionCollection = new KActionCollection( this );
+  mActionCollection->setObjectName( "ActionCollection" );
   QTimer::singleShot( 0, this, SLOT( createActions() ) );
 }
 
@@ -71,18 +75,18 @@ ExtensionManager::~ExtensionManager()
 void ExtensionManager::restoreSettings()
 {
   const QStringList activeExtensions = KABPrefs::instance()->activeExtensions();
-
-  typedef QMap<QString, ExtensionData>::ConstIterator ConstIterator;
-  for ( ConstIterator it = mExtensionMap.begin(), end = mExtensionMap.end(); it != end; ++it ) {
-    if ( activeExtensions.contains( it.data().identifier ) ) {
-      KToggleAction *action = static_cast<KToggleAction*>( it.data().action );
+  Q_FOREACH ( const ExtensionData data, mExtensionMap ) {
+      if ( activeExtensions.contains( data.identifier ) ) {
+      KToggleAction *action = static_cast<KToggleAction*>( data.action );
       if ( action )
         action->setChecked( true );
-      setExtensionActive( it.data().identifier, true );
+      setExtensionActive( data.identifier, true );
     }
   }
-  const QValueList<int> sizes = KABPrefs::instance()->extensionsSplitterSizes();
-  mSplitter->setSizes( sizes );
+  const QList<int> sizes = KABPrefs::instance()->extensionsSplitterSizes();
+  if (!sizes.isEmpty()) {
+    mSplitter->setSizes( sizes );
+  }
 }
 
 void ExtensionManager::saveSettings()
@@ -97,7 +101,7 @@ void ExtensionManager::reconfigure()
   createExtensionWidgets();
   createActions();
   restoreSettings();
-  mExtensionBar->setShown( !mActiveExtensions.isEmpty() );
+  updateExtensionBarVisibility();
 }
 
 bool ExtensionManager::isQuickEditVisible() const
@@ -107,9 +111,9 @@ bool ExtensionManager::isQuickEditVisible() const
 
 void ExtensionManager::setSelectionChanged()
 {
-  for ( QStringList::ConstIterator it = mActiveExtensions.begin(), end = mActiveExtensions.end(); it != end; ++it ) {
-    if ( mExtensionMap.contains( *it ) && mExtensionMap[*it].widget )
-      mExtensionMap[*it].widget->contactsSelectionChanged();
+  foreach ( const QString i, mActiveExtensions ) {
+    if ( mExtensionMap.contains( i ) && mExtensionMap[i].widget )
+      mExtensionMap[i].widget->contactsSelectionChanged();
   }
 }
 
@@ -117,9 +121,19 @@ void ExtensionManager::activationToggled( const QString &extid )
 {
   if ( !mExtensionMap.contains( extid ) )
     return;
-  const ExtensionData data = mExtensionMap[ extid ];
-  const bool activated = data.action->isChecked();
-  setExtensionActive( extid, activated );
+  setExtensionActive( extid, mExtensionMap[ extid ].action->isChecked() );
+}
+
+void ExtensionManager::updateExtensionBarVisibility()
+{
+  foreach ( const QString i, mActiveExtensions ) {
+    if ( mExtensionMap[i].widget && !mExtensionMap[i].isDetailsExtension ) {
+      mExtensionBar->setVisible( true );
+      return;
+    }
+  }
+
+  mExtensionBar->setVisible( false );
 }
 
 void ExtensionManager::setExtensionActive( const QString& extid, bool active )
@@ -136,50 +150,47 @@ void ExtensionManager::setExtensionActive( const QString& extid, bool active )
         mActiveDetailsWidget = data.widget;
         emit detailsWidgetActivated( data.widget );
       } else {
-          data.widget->show();
+          data.widget->setVisible( true );
       }
       data.widget->contactsSelectionChanged();
     }
   } else {
-    mActiveExtensions.remove( extid );
-    if ( data.widget && !data.isDetailsExtension ) {
-      data.widget->hide();
-    }
+    mActiveExtensions.removeAll( extid );
     if ( data.isDetailsExtension ) {
       mActiveDetailsWidget = 0;
       emit detailsWidgetDeactivated( data.widget );
+    } else {
+        data.widget->setVisible( false );
     }
   }
-  mExtensionBar->setShown( !mActiveExtensions.isEmpty() );
+  updateExtensionBarVisibility();
 }
 
 void ExtensionManager::createActions()
 {
   mCore->guiClient()->unplugActionList( "extensions_list" );
-  mActionList.setAutoDelete( true );
+  qDeleteAll( mActionList );
   mActionList.clear();
-  mActionList.setAutoDelete( false );
 
   delete mMapper;
-  mMapper = new QSignalMapper( this, "SignalMapper" );
+  mMapper = new QSignalMapper( this );
+  mMapper->setObjectName( "SignalMapper" );
   connect( mMapper, SIGNAL( mapped( const QString& ) ),
            this, SLOT( activationToggled( const QString& ) ) );
 
-  ExtensionData::List::ConstIterator it;
-  for ( QMap<QString, ExtensionData>::Iterator it = mExtensionMap.begin(), end = mExtensionMap.end(); it != end; ++it ) {
-    ExtensionData& data = it.data();
-    data.action = new KToggleAction( data.title, 0, mMapper, SLOT( map() ),
-                                               mActionCollection,
-                                               QString( data.identifier + "_extension" ).latin1() );
+  foreach ( const QString i, mExtensionMap.keys() ) {
+    ExtensionData& data = mExtensionMap[i];
+    data.action = mActionCollection->add<KToggleAction>( QString( data.identifier + "_extension" ) );
+    data.action->setText( data.title );
+    connect( data.action, SIGNAL(triggered(bool) ), mMapper, SLOT( map() ));
     mMapper->setMapping( data.action, data.identifier );
     mActionList.append( data.action );
-
     if ( mActiveExtensions.contains( data.identifier ) )
       data.action->setChecked( true );
   }
 
-  mActionList.append( new KActionSeparator( mActionCollection ) );
   mCore->guiClient()->plugActionList( "extensions_list", mActionList );
+  updateExtensionBarVisibility();
 }
 
 QWidget* ExtensionManager::activeDetailsWidget() const
@@ -190,17 +201,15 @@ QWidget* ExtensionManager::activeDetailsWidget() const
 void ExtensionManager::createExtensionWidgets()
 {
   // clean up
-  for ( QMap<QString, ExtensionData>::ConstIterator it = mExtensionMap.begin(), end = mExtensionMap.end(); it != end; ++it ) {
-    delete it.data().widget;
-  }
+  foreach ( const ExtensionData i, mExtensionMap )
+    delete i.widget;
   mExtensionMap.clear();
 
-  KAB::ExtensionWidget *wdg = 0;
-
   {
     // add addressee editor as default
-    wdg = new AddresseeEditorExtension( mCore, mDetailsStack );
-    wdg->hide();
+    KAB::ExtensionWidget *wdg = new AddresseeEditorExtension( mCore, 0 );
+    mDetailsStack->addWidget( wdg );
+    wdg->setVisible( false );
 
     connect( wdg, SIGNAL( modified( const KABC::Addressee::List& ) ),
              SIGNAL( modified( const KABC::Addressee::List& ) ) );
@@ -216,29 +225,30 @@ void ExtensionManager::createExtensionWidgets()
   }
 
   // load the other extensions
-  const KTrader::OfferList plugins = KTrader::self()->query( "KAddressBook/Extension",
+  const KService::List plugins = KServiceTypeTrader::self()->query( "KAddressBook/Extension",
     QString( "[X-KDE-KAddressBook-ExtensionPluginVersion] == %1" ).arg( KAB_EXTENSIONWIDGET_PLUGIN_VERSION ) );
 
-  KTrader::OfferList::ConstIterator it;
-  for ( it = plugins.begin(); it != plugins.end(); ++it ) {
-    KLibFactory *factory = KLibLoader::self()->factory( (*it)->library().latin1() );
+  foreach ( KService::Ptr pluginService, plugins ) {
+    KPluginFactory *factory = KPluginLoader( *pluginService ).factory();
     if ( !factory ) {
-      kdDebug(5720) << "ExtensionManager::loadExtensions(): Factory creation failed" << endl;
+      kDebug(5720) <<"ExtensionManager::loadExtensions(): Factory creation failed";
       continue;
     }
 
-    KAB::ExtensionFactory *extensionFactory = static_cast<KAB::ExtensionFactory*>( factory );
+    KAB::ExtensionFactory *extensionFactory = qobject_cast<KAB::ExtensionFactory*>( factory );
 
     if ( !extensionFactory ) {
-      kdDebug(5720) << "ExtensionManager::loadExtensions(): Cast failed" << endl;
+      kDebug(5720) <<"ExtensionManager::loadExtensions(): Cast failed";
       continue;
     }
 
-    wdg = extensionFactory->extension( mCore, mSplitter );
+    KAB::ExtensionWidget *wdg = extensionFactory->extension( mCore, 0 );
     if ( wdg ) {
+      mSplitter->addWidget( wdg );
       if ( wdg->identifier() == "distribution_list_editor_ng" )
-          mSplitter->moveToFirst( wdg );
-      wdg->hide();
+          mSplitter->insertWidget( 0, wdg );
+      wdg->updateGeometry();
+      wdg->setVisible( false );
       connect( wdg, SIGNAL( modified( const KABC::Addressee::List& ) ),
                SIGNAL( modified( const KABC::Addressee::List& ) ) );
       connect( wdg, SIGNAL( deleted( const QStringList& ) ),
