@@ -22,7 +22,7 @@
 */
 
 #include <kabc/geo.h>
-#include <kaccelmanager.h>
+#include <kacceleratormanager.h>
 #include <kcombobox.h>
 #include <kdebug.h>
 #include <kiconloader.h>
@@ -30,59 +30,68 @@
 #include <knuminput.h>
 #include <kstandarddirs.h>
 
-#include <qcheckbox.h>
-#include <qfile.h>
-#include <qgroupbox.h>
-#include <qlabel.h>
-#include <qlayout.h>
-#include <qlistbox.h>
-#include <qpainter.h>
-#include <qpixmap.h>
-#include <qpushbutton.h>
-#include <qregexp.h>
-#include <qstring.h>
+#include <QCheckBox>
+#include <QFile>
+#include <QGroupBox>
+#include <QLabel>
+#include <QPainter>
+#include <QPixmap>
+#include <QPushButton>
+#include <QRegExp>
+#include <QString>
+#include <QGridLayout>
 
 #include "geowidget.h"
 
-GeoWidget::GeoWidget( KABC::AddressBook *ab, QWidget *parent, const char *name )
-  : KAB::ContactEditorWidget( ab, parent, name ), mReadOnly( false )
+GeoWidget::GeoWidget( KABC::AddressBook *ab, QWidget *parent )
+  : KAB::ContactEditorWidget( ab, parent ), mReadOnly( false )
 {
   QLabel *label = 0;
 
-  QGridLayout *topLayout = new QGridLayout( this, 4, 3 );
+  QGridLayout *topLayout = new QGridLayout( this );
   topLayout->setMargin( KDialog::marginHint() );
   topLayout->setSpacing( KDialog::spacingHint() );
 
   label = new QLabel( this );
-  label->setPixmap( KGlobal::iconLoader()->loadIcon( "package_network",
-                    KIcon::Desktop, KIcon::SizeMedium ) );
+  label->setPixmap( KIconLoader::global()->loadIcon( "applications-internet",
+                    KIconLoader::Desktop, KIconLoader::SizeMedium ) );
   label->setAlignment( Qt::AlignTop );
-  topLayout->addMultiCellWidget( label, 0, 3, 0, 0 );
+  topLayout->addWidget( label, 0, 0, 4, 1);
 
   mGeoIsValid = new QCheckBox( i18n( "Use geo data" ), this );
-  topLayout->addMultiCellWidget( mGeoIsValid, 0, 0, 1, 2 );
+  topLayout->addWidget( mGeoIsValid, 0, 1, 1, 2 );
 
   label = new QLabel( i18n( "Latitude:" ), this );
   topLayout->addWidget( label, 1, 1 );
 
-  mLatitudeBox = new KDoubleSpinBox( -90, 90, 1, 0, 6, this );
+  mLatitudeBox = new QDoubleSpinBox( this );
+  mLatitudeBox->setMinimum( -90 );
+  mLatitudeBox->setMaximum( 90 );
+  mLatitudeBox->setSingleStep( 1 );
+  mLatitudeBox->setValue( 0 );
+  mLatitudeBox->setDecimals( 6 );
   mLatitudeBox->setEnabled( false );
-  mLatitudeBox->setSuffix( "" );
+  mLatitudeBox->setSuffix( QString::fromUtf8( "°" ) );
   topLayout->addWidget( mLatitudeBox, 1, 2 );
   label->setBuddy( mLatitudeBox );
 
   label = new QLabel( i18n( "Longitude:" ), this );
   topLayout->addWidget( label, 2, 1 );
 
-  mLongitudeBox = new KDoubleSpinBox( -180, 180, 1, 0, 6, this );
+  mLongitudeBox = new QDoubleSpinBox( this );
+  mLongitudeBox->setMinimum( -180 );
+  mLongitudeBox->setMaximum( 180 );
+  mLongitudeBox->setSingleStep( 1 );
+  mLongitudeBox->setValue( 0 );
+  mLongitudeBox->setDecimals( 6 );
   mLongitudeBox->setEnabled( false );
-  mLongitudeBox->setSuffix( "" );
+  mLongitudeBox->setSuffix( QString::fromUtf8( "°" ) );
   topLayout->addWidget( mLongitudeBox, 2, 2 );
   label->setBuddy( mLongitudeBox );
 
   mExtendedButton = new QPushButton( i18n( "Edit Geo Data..." ), this );
   mExtendedButton->setEnabled( false );
-  topLayout->addMultiCellWidget( mExtendedButton, 3, 3, 1, 2 );
+  topLayout->addWidget( mExtendedButton, 3, 1, 1, 2 );
 
   connect( mLatitudeBox, SIGNAL( valueChanged( double ) ),
            SLOT( setModified() ) );
@@ -157,68 +166,95 @@ void GeoWidget::editGeoData()
 
 
 
-GeoDialog::GeoDialog( QWidget *parent, const char *name )
-  : KDialogBase( Plain, i18n( "Geo Data Input" ), Ok | Cancel, Ok,
-                 parent, name, true, true ),
+GeoDialog::GeoDialog( QWidget *parent )
+  : KDialog( parent ),
     mUpdateSexagesimalInput( true )
 {
-  QFrame *page = plainPage();
+  setCaption( i18n( "Geo Data Input" ) );
+  setButtons( Ok | Cancel );
+  setDefaultButton( Ok );
+  showButtonSeparator( true );
+  setModal( true );
 
-  QGridLayout *topLayout = new QGridLayout( page, 2, 2, marginHint(),
-                                            spacingHint() );
+  QFrame *page = new QFrame(this);
+  setMainWidget( page );
+
+  QGridLayout *topLayout = new QGridLayout( page );
+  topLayout->setSpacing( spacingHint() );
+  topLayout->setMargin( 0 );
   topLayout->setRowStretch( 1, 1 );
 
   mMapWidget = new GeoMapWidget( page );
-  topLayout->addMultiCellWidget( mMapWidget, 0, 1, 0, 0 );
+  topLayout->addWidget( mMapWidget, 0, 0, 2, 1);
 
   mCityCombo = new KComboBox( page );
   topLayout->addWidget( mCityCombo, 0, 1 );
 
-  QGroupBox *sexagesimalGroup = new QGroupBox( 0, Vertical, i18n( "Sexagesimal" ), page );
-  QGridLayout *sexagesimalLayout = new QGridLayout( sexagesimalGroup->layout(),
-                                                    2, 5, spacingHint() );
+  QGroupBox *sexagesimalGroup = new QGroupBox( i18n( "Sexagesimal" ), page );
+  QGridLayout *sexagesimalLayout = new QGridLayout();
+  sexagesimalGroup->setLayout( sexagesimalLayout );
+  sexagesimalLayout->setSpacing( spacingHint() );
 
   QLabel *label = new QLabel( i18n( "Latitude:" ), sexagesimalGroup );
   sexagesimalLayout->addWidget( label, 0, 0 );
 
-  mLatDegrees = new QSpinBox( 0, 90, 1, sexagesimalGroup );
-  mLatDegrees->setSuffix( "" );
+  mLatDegrees = new QSpinBox( sexagesimalGroup );
+  mLatDegrees->setMinimum( 0 );
+  mLatDegrees->setMaximum( 90 );
+  mLatDegrees->setValue( 1 );
+  mLatDegrees->setSuffix( QString::fromUtf8( "°" ) );
   mLatDegrees->setWrapping( false );
   label->setBuddy( mLatDegrees );
   sexagesimalLayout->addWidget( mLatDegrees, 0, 1 );
 
-  mLatMinutes = new QSpinBox( 0, 59, 1, sexagesimalGroup );
+  mLatMinutes = new QSpinBox( sexagesimalGroup );
+  mLatMinutes->setMinimum( 0 );
+  mLatMinutes->setMaximum( 59 );
+  mLatMinutes->setValue( 1 );
+
   mLatMinutes->setSuffix( "'" );
   sexagesimalLayout->addWidget( mLatMinutes, 0, 2 );
 
-  mLatSeconds = new QSpinBox( 0, 59, 1, sexagesimalGroup );
+  mLatSeconds = new QSpinBox( sexagesimalGroup );
+  mLatSeconds->setMinimum( 0 );
+  mLatSeconds->setMaximum( 59 );
+  mLatSeconds->setValue( 1 );
   mLatSeconds->setSuffix( "\"" );
   sexagesimalLayout->addWidget( mLatSeconds, 0, 3 );
 
   mLatDirection = new KComboBox( sexagesimalGroup );
-  mLatDirection->insertItem( i18n( "North" ) );
-  mLatDirection->insertItem( i18n( "South" ) );
+  mLatDirection->addItem( i18n( "North" ) );
+  mLatDirection->addItem( i18n( "South" ) );
   sexagesimalLayout->addWidget( mLatDirection, 0, 4 );
 
   label = new QLabel( i18n( "Longitude:" ), sexagesimalGroup );
   sexagesimalLayout->addWidget( label, 1, 0 );
 
-  mLongDegrees = new QSpinBox( 0, 180, 1, sexagesimalGroup );
-  mLongDegrees->setSuffix( "" );
+  mLongDegrees = new QSpinBox( sexagesimalGroup );
+  mLongDegrees->setMinimum( 0 );
+  mLongDegrees->setMaximum( 180 );
+  mLongDegrees->setValue( 1 );
+  mLongDegrees->setSuffix( QString::fromUtf8( "°" ) );
   label->setBuddy( mLongDegrees );
   sexagesimalLayout->addWidget( mLongDegrees, 1, 1 );
 
-  mLongMinutes = new QSpinBox( 0, 59, 1, sexagesimalGroup );
+  mLongMinutes = new QSpinBox( sexagesimalGroup );
+  mLongMinutes->setMinimum( 0 );
+  mLongMinutes->setMaximum( 59 );
+  mLongMinutes->setValue( 1 );
   mLongMinutes->setSuffix( "'" );
   sexagesimalLayout->addWidget( mLongMinutes, 1, 2 );
 
-  mLongSeconds = new QSpinBox( 0, 59, 1, sexagesimalGroup );
+  mLongSeconds = new QSpinBox( sexagesimalGroup );
+  mLongSeconds->setMinimum( 0 );
+  mLongSeconds->setMaximum( 59 );
+  mLongSeconds->setValue( 1 );
   mLongSeconds->setSuffix( "\"" );
   sexagesimalLayout->addWidget( mLongSeconds, 1, 3 );
 
   mLongDirection = new KComboBox( sexagesimalGroup );
-  mLongDirection->insertItem( i18n( "East" ) );
-  mLongDirection->insertItem( i18n( "West" ) );
+  mLongDirection->addItem( i18n( "East" ) );
+  mLongDirection->addItem( i18n( "West" ) );
   sexagesimalLayout->addWidget( mLongDirection, 1, 4 );
 
   topLayout->addWidget( sexagesimalGroup, 1, 1 );
@@ -280,12 +316,12 @@ void GeoDialog::sexagesimalInputChanged()
   mLatitude = (double)( mLatDegrees->value() + (double)mLatMinutes->value() /
                         60 + (double)mLatSeconds->value() / 3600 );
 
-  mLatitude *= ( mLatDirection->currentItem() == 1 ? -1 : 1 );
+  mLatitude *= ( mLatDirection->currentIndex() == 1 ? -1 : 1 );
 
   mLongitude = (double)( mLongDegrees->value() + (double)mLongMinutes->value() /
                          60 + (double)mLongSeconds->value() / 3600 );
 
-  mLongitude *= ( mLongDirection->currentItem() == 1 ? -1 : 1 );
+  mLongitude *= ( mLongDirection->currentIndex() == 1 ? -1 : 1 );
 
   mUpdateSexagesimalInput = false;
 
@@ -302,7 +338,7 @@ void GeoDialog::geoMapChanged()
 
 void GeoDialog::cityInputChanged()
 {
-  if ( mCityCombo->currentItem() != 0 ) {
+  if ( mCityCombo->currentIndex() != 0 ) {
     GeoData data = mGeoDataMap[ mCityCombo->currentText() ];
     mLatitude = data.latitude;
     mLongitude = data.longitude;
@@ -345,7 +381,7 @@ void GeoDialog::updateInputs()
     mLatMinutes->setValue( minutes );
     mLatSeconds->setValue( seconds );
 
-    mLatDirection->setCurrentItem( mLatitude < 0 ? 1 : 0 );
+    mLatDirection->setCurrentIndex( mLatitude < 0 ? 1 : 0 );
 
     degrees = (int)( longitude * 1 );
     minutes = (int)( ( longitude - degrees ) * 60 );
@@ -354,15 +390,15 @@ void GeoDialog::updateInputs()
     mLongDegrees->setValue( degrees );
     mLongMinutes->setValue( minutes );
     mLongSeconds->setValue( seconds );
-    mLongDirection->setCurrentItem( mLongitude < 0 ? 1 : 0 );
+    mLongDirection->setCurrentIndex( mLongitude < 0 ? 1 : 0 );
   }
   mUpdateSexagesimalInput = true;
 
   int pos = nearestCity( mLongitude, mLatitude );
   if ( pos != -1 )
-    mCityCombo->setCurrentItem( pos + 1 );
+    mCityCombo->setCurrentIndex( pos + 1 );
   else
-    mCityCombo->setCurrentItem( 0 );
+    mCityCombo->setCurrentIndex( 0 );
 
   mCityCombo->blockSignals( false );
   mLatDegrees->blockSignals( false );
@@ -380,9 +416,9 @@ void GeoDialog::loadCityList()
   mCityCombo->clear();
   mGeoDataMap.clear();
 
-  QFile file( locate( "data", "kaddressbook/zone.tab" ) );
+  QFile file( KStandardDirs::locate( "data", "kaddressbook/zone.tab" ) );
 
-  if ( file.open( IO_ReadOnly ) ) {
+  if ( file.open( QIODevice::ReadOnly ) ) {
     QTextStream s( &file );
 
     QString line, country;
@@ -390,27 +426,27 @@ void GeoDialog::loadCityList()
     QRegExp name( "[^\\s]+/[^\\s]+" );
     int pos;
 
-    while ( !s.eof() ) {
-      line = s.readLine().stripWhiteSpace();
+    while ( !s.atEnd() ) {
+      line = s.readLine().trimmed();
       if ( line.isEmpty() || line[ 0 ] == '#' )
         continue;
 
       country = line.left( 2 );
       QString c, n;
-      pos = coord.search( line, 0 );
+      pos = coord.indexIn( line, 0 );
       if ( pos >= 0 )
         c = line.mid( pos, coord.matchedLength() );
 
-      pos = name.search(line, pos);
+      pos = name.indexIn(line, pos);
       if ( pos > 0 ) {
-        n = line.mid( pos, name.matchedLength() ).stripWhiteSpace();
+        n = line.mid( pos, name.matchedLength() ).trimmed();
         n.replace( '_', " " );
       }
 
       if ( !c.isEmpty() && !n.isEmpty() ) {
-        pos = c.find( "+", 1 );
+        pos = c.indexOf( "+", 1 );
         if ( pos < 0 )
-          pos = c.find( "-", 1 );
+          pos = c.indexOf( "-", 1 );
         if ( pos > 0 ) {
           GeoData data;
           data.latitude = calculateCoordinate( c.left( pos ) );
@@ -423,7 +459,7 @@ void GeoDialog::loadCityList()
     }
     QStringList items( mGeoDataMap.keys() );
     items.prepend( i18n( "Undefined" ) );
-    mCityCombo->insertStringList( items );
+    mCityCombo->addItems( items );
 
     file.close();
   }
@@ -482,10 +518,10 @@ int GeoDialog::nearestCity( double x, double y ) const
 }
 
 
-GeoMapWidget::GeoMapWidget( QWidget *parent, const char *name )
-  : QWidget( parent, name ), mLatitude( 0 ), mLongitude( 0 )
+GeoMapWidget::GeoMapWidget( QWidget *parent )
+  : QWidget( parent ), mLatitude( 0 ), mLongitude( 0 )
 {
-  setBackgroundMode( NoBackground );
+  setAttribute(Qt::WA_NoSystemBackground, true);
 
   setFixedSize( 400, 200 );
 
@@ -537,12 +573,13 @@ void GeoMapWidget::paintEvent( QPaintEvent* )
 
   QPixmap pm( w, h );
   QPainter p;
-  p.begin( &pm, this );
+  p.begin( &pm );
+  p.initFrom( this );
 
   p.setPen( QColor( 255, 0, 0 ) );
   p.setBrush( QColor( 255, 0, 0 ) );
 
-  QPixmap world( locate( "data", "kaddressbook/pics/world.jpg" ) );
+  QPixmap world( KStandardDirs::locate( "data", "kaddressbook/pics/world.jpg" ) );
   p.drawPixmap( 0, 0, world );
 
   double latMid = h / 2;
@@ -556,7 +593,8 @@ void GeoMapWidget::paintEvent( QPaintEvent* )
   p.drawEllipse( x, y, 4, 4 );
 
   p.end();
-  bitBlt( this, 0, 0, &pm );
+  QPainter thisPainter(this);
+  thisPainter.drawPixmap( 0, 0, pm );
 }
 
 #include "geowidget.moc"
