@@ -12,9 +12,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
     As a special exception, permission is given to link this program
     with any edition of Qt, and distribute the resulting executable,
@@ -31,28 +31,23 @@
 #include <kio/netaccess.h>
 #include <klocale.h>
 #include <kmessagebox.h>
-#include <kurldrag.h>
-#include <libkdepim/kpixmapregionselectordialog.h>
+#include <kpixmapregionselectordialog.h>
 
-#include <qapplication.h>
-#include <qdragobject.h>
-#include <qeventloop.h>
-#include <qgroupbox.h>
-#include <qlabel.h>
-#include <qlayout.h>
-#include <qpixmap.h>
-#include <qpopupmenu.h>
-
-#include <unistd.h>
+#include <QtGui/QGroupBox>
+#include <QtGui/QImage>
+#include <QtGui/QMenu>
+#include <QtGui/QMouseEvent>
+#include <QtGui/QPixmap>
+#include <QtGui/QVBoxLayout>
 
 #include "imagewidget.h"
 
 ImageLoader::ImageLoader( QWidget *parent )
-  : QObject( 0, "ImageLoader" ), mParent( parent )
+  : QObject( 0 ), mParent( parent )
 {
 }
 
-KABC::Picture ImageLoader::loadPicture( const KURL &url, bool *ok )
+KABC::Picture ImageLoader::loadPicture( const KUrl &url, bool *ok )
 {
   KABC::Picture picture;
   QString tempFile;
@@ -80,20 +75,19 @@ KABC::Picture ImageLoader::loadPicture( const KURL &url, bool *ok )
     return picture;
   }
 
-  QPixmap pixmap = picture.data();
+  QPixmap pixmap = QPixmap::fromImage( picture.data() );
 
-  QPixmap selectedPixmap = KPIM::KPixmapRegionSelectorDialog::getSelectedImage( pixmap, 100, 140, mParent );
-  if ( selectedPixmap.isNull() ) {
+  image = KPixmapRegionSelectorDialog::getSelectedImage( pixmap, 100, 140, mParent );
+  if ( image.isNull() ) {
     (*ok) = false;
     return picture;
   }
 
-  image = selectedPixmap;
   if ( image.height() != 140 || image.width() != 100 ) {
     if ( image.height() > image.width() )
-      image = image.scaleHeight( 140 );
+      image = image.scaledToHeight( 140 );
     else
-      image = image.scaleWidth( 100 );
+      image = image.scaledToWidth( 100 );
   }
 
   picture.setData( image );
@@ -102,12 +96,12 @@ KABC::Picture ImageLoader::loadPicture( const KURL &url, bool *ok )
   return picture;
 }
 
-
-ImageButton::ImageButton( const QString &title, QWidget *parent )
-  : QPushButton( title, parent ),
+ImageButton::ImageButton( QWidget *parent )
+  : QPushButton( parent ),
     mReadOnly( false ), mImageLoader( 0 )
 {
   setAcceptDrops( true );
+  setIconSize( QSize( 100, 140 ) );
 
   connect( this, SIGNAL( clicked() ), SLOT( load() ) );
 }
@@ -120,7 +114,7 @@ void ImageButton::setReadOnly( bool readOnly )
 void ImageButton::setPicture( const KABC::Picture &picture )
 {
   mPicture = picture;
-  updateGUI();
+  updateGui();
 }
 
 KABC::Picture ImageButton::picture() const
@@ -136,30 +130,26 @@ void ImageButton::setImageLoader( ImageLoader *loader )
 void ImageButton::startDrag()
 {
   if ( !mPicture.data().isNull() ) {
-    QImageDrag *drag = new QImageDrag( mPicture.data(), this );
-    drag->dragCopy();
+    QDrag *drag = new QDrag( this );
+    drag->setMimeData( new QMimeData() );
+    drag->mimeData()->setImageData( mPicture.data() );
+    drag->start();
   }
 }
 
-void ImageButton::updateGUI()
+void ImageButton::updateGui()
 {
-  if ( mPicture.data().isNull() )
-    setPixmap( KGlobal::iconLoader()->iconPath( "personal", KIcon::Desktop ) );
-  else
-    setPixmap( mPicture.data() );
+  if ( mPicture.data().isNull() ) {
+    setIcon( KIcon( "user-identity" ) );
+  } else {
+    setIcon( QPixmap::fromImage( mPicture.data() ) );
+  }
 }
 
 void ImageButton::dragEnterEvent( QDragEnterEvent *event )
 {
-  bool accepted = false;
-
-  if ( QImageDrag::canDecode( event ) )
-    accepted = true;
-
-  if ( QUriDrag::canDecode( event ) )
-    accepted = true;
-
-  event->accept( accepted );
+  const QMimeData *md = event->mimeData();
+  event->setAccepted( md->hasImage() || md->hasUrls());
 }
 
 void ImageButton::dropEvent( QDropEvent *event )
@@ -167,31 +157,24 @@ void ImageButton::dropEvent( QDropEvent *event )
   if ( mReadOnly )
     return;
 
-  if ( QImageDrag::canDecode( event ) ) {
-    QPixmap pm;
-
-    if ( QImageDrag::decode( event, pm ) ) {
-      mPicture.setData( pm.convertToImage() );
-      updateGUI();
-      emit changed();
-    }
+  const QMimeData *md = event->mimeData();
+  if ( md->hasImage() ) {
+    QImage image = qvariant_cast<QImage>(md->imageData());
+    mPicture.setData( image );
+    updateGui();
+    emit changed();
   }
 
-  if ( QUriDrag::canDecode( event ) ) {
-    KURL::List urls;
-    if ( KURLDrag::decode( event, urls ) ) {
-      if ( urls.isEmpty() ) { // oops, no data
-        event->accept( false );
-        return;
-      }
-    }
-
+  KUrl::List urls = KUrl::List::fromMimeData( md );
+  if ( urls.isEmpty() ) { // oops, no data
+    event->setAccepted( false );
+  } else {
     if ( mImageLoader ) {
       bool ok = false;
-      KABC::Picture pic = mImageLoader->loadPicture( urls[ 0 ], &ok );
+      KABC::Picture pic = mImageLoader->loadPicture( urls.first(), &ok );
       if ( ok ) {
         mPicture = pic;
-        updateGUI();
+        updateGui();
         emit changed();
       }
     }
@@ -206,7 +189,7 @@ void ImageButton::mousePressEvent( QMouseEvent *event )
 
 void ImageButton::mouseMoveEvent( QMouseEvent *event )
 {
-  if ( (event->state() & LeftButton) &&
+  if ( (event->buttons() & Qt::LeftButton) &&
        (event->pos() - mDragStartPos).manhattanLength() >
        KGlobalSettings::dndEventDelay() ) {
     startDrag();
@@ -215,21 +198,21 @@ void ImageButton::mouseMoveEvent( QMouseEvent *event )
 
 void ImageButton::contextMenuEvent( QContextMenuEvent *event )
 {
-  QPopupMenu menu( this );
-  menu.insertItem( i18n( "Reset" ), this, SLOT( clear() ) );
+  QMenu menu;
+  menu.addAction( i18n( "Reset" ), this, SLOT( clear() ) );
   menu.exec( event->globalPos() );
 }
 
 void ImageButton::load()
 {
-  KURL url = KFileDialog::getOpenURL( QString(), KImageIO::pattern(), this );
+  KUrl url = KFileDialog::getOpenUrl( QString(), KImageIO::pattern(), this );
   if ( url.isValid() ) {
     if ( mImageLoader ) {
       bool ok = false;
       KABC::Picture pic = mImageLoader->loadPicture( url, &ok );
       if ( ok ) {
         mPicture = pic;
-        updateGUI();
+        updateGui();
         emit changed();
       }
     }
@@ -239,25 +222,28 @@ void ImageButton::load()
 void ImageButton::clear()
 {
   mPicture = KABC::Picture();
-  updateGUI();
+  updateGui();
 
   emit changed();
 }
 
-ImageBaseWidget::ImageBaseWidget( const QString &title,
-                                  QWidget *parent, const char *name )
-  : QWidget( parent, name ), mReadOnly( false )
+ImageBaseWidget::ImageBaseWidget( const QString &title, QWidget *parent )
+  : QWidget( parent ), mReadOnly( false )
 {
   mImageLoader = new ImageLoader( this );
 
-  QVBoxLayout *topLayout = new QVBoxLayout( this, KDialog::marginHint(),
-                                            KDialog::spacingHint() );
-  QGroupBox *box = new QGroupBox( 0, Qt::Vertical, title, this );
-  QVBoxLayout *layout = new QVBoxLayout( box->layout(), KDialog::spacingHint() );
+  QVBoxLayout *topLayout = new QVBoxLayout( this );
+  topLayout->setMargin( KDialog::marginHint() );
+  topLayout->setSpacing( KDialog::spacingHint() );
 
-  mImageButton = new ImageButton( i18n( "Picture" ), box );
-  mImageButton->setFixedSize( 100, 140 );
+  QGroupBox *box = new QGroupBox( title, this );
+  QVBoxLayout *layout = new QVBoxLayout( box );
+  layout->setSpacing( KDialog::spacingHint() );
+
+  mImageButton = new ImageButton( box );
+  mImageButton->setFixedSize( 120, 160 );
   mImageButton->setImageLoader( mImageLoader );
+
   layout->addWidget( mImageButton );
 
   topLayout->addWidget( box );
@@ -288,11 +274,12 @@ KABC::Picture ImageBaseWidget::image() const
 }
 
 
-ImageWidget::ImageWidget( KABC::AddressBook *ab, QWidget *parent, const char *name )
-  : KAB::ContactEditorWidget( ab, parent, name )
+ImageWidget::ImageWidget( KABC::AddressBook *ab, QWidget *parent )
+  : KAB::ContactEditorWidget( ab, parent )
 {
-  QHBoxLayout *layout = new QHBoxLayout( this, KDialog::marginHint(),
-                                         KDialog::spacingHint() );
+  QHBoxLayout *layout = new QHBoxLayout( this );
+  layout->setSpacing( KDialog::spacingHint() );
+  layout->setMargin( 0 );
 
   mPhotoWidget = new ImageBaseWidget( KABC::Addressee::photoLabel(), this );
   layout->addWidget( mPhotoWidget );
