@@ -21,13 +21,17 @@
     without including the source code for Qt in the source distribution.
 */
 
-#include <qapplication.h>
-#include <qbuttongroup.h>
-#include <qevent.h>
-#include <qlayout.h>
-#include <qpushbutton.h>
-#include <qstring.h>
-#include <qstyle.h>
+#include <QApplication>
+#include <QButtonGroup>
+#include <QGroupBox>
+#include <QList>
+#include <QPushButton>
+#include <QResizeEvent>
+#include <QString>
+#include <QStyle>
+#include <QStyleOption>
+#include <QTimer>
+#include <QVBoxLayout>
 
 #include <kabc/addressbook.h>
 #include <kabc/field.h>
@@ -39,6 +43,23 @@
 
 #include "jumpbuttonbar.h"
 
+static bool localAwareLessThan( const QString &left, const QString &right )
+{
+  return ( QString::localeAwareCompare( left, right ) < 0 );
+}
+
+static QSize calculateButtonSize( QWidget *parent )
+{
+  QStyleOption opt;
+  QFontMetrics fm = parent->fontMetrics();
+
+  const QSize buttonSize = parent->style()->sizeFromContents( QStyle::CT_PushButton, &opt,
+                                    fm.size( Qt::TextShowMnemonic, " X - X " )
+                                    .expandedTo( QApplication::globalStrut() ) );
+
+  return buttonSize;
+}
+
 class JumpButton : public QPushButton
 {
   public:
@@ -55,28 +76,33 @@ JumpButton::JumpButton( const QString &firstChar, const QString &lastChar,
                         QWidget *parent )
   : QPushButton( "", parent ), mChar( firstChar )
 {
-  setToggleButton( true );
+  setCheckable( true );
+
   if ( !lastChar.isEmpty() )
-    setText( QString( "%1 - %2" ).arg( firstChar.upper() ).arg( lastChar.upper() ) );
+    setText( QString( "%1 - %2" ).arg( firstChar.toUpper() ).arg( lastChar.toUpper() ) );
   else
-    setText( firstChar.upper() );
+    setText( firstChar.toUpper() );
+
+  static QSize buttonSize;
+  if ( !buttonSize.isValid() )
+    buttonSize = calculateButtonSize( this );
+
+  setFixedWidth( buttonSize.width() + 10 );  // +10 for buggy oxygen style
+  setMinimumHeight( 1 );
 }
 
+
 JumpButtonBar::JumpButtonBar( KAB::Core *core, QWidget *parent, const char *name )
-  : QWidget( parent, name ), mCore( core )
+  : QWidget( parent ), mCore( core ), mButtonsUpdated( true )
 {
-  setMinimumSize( 1, 1 );
+  setObjectName( name );
 
-  QVBoxLayout *layout = new QVBoxLayout( this, 0, 0 );
-  layout->setAlignment( Qt::AlignTop );
-  layout->setAutoAdd( true );
-  layout->setResizeMode( QLayout::FreeResize );
+  mLayout = new QVBoxLayout( this );
+  mLayout->setMargin( 5 );
+  mLayout->setSpacing( 5 );
 
-  mGroupBox = new QButtonGroup( 1, Qt::Horizontal, this );
-  mGroupBox->setExclusive( true );
-  mGroupBox->layout()->setSpacing( 0 );
-  mGroupBox->layout()->setMargin( 0 );
-  mGroupBox->setFrameStyle( QFrame::NoFrame );
+  mButtonGroup = new QButtonGroup;
+  mButtonGroup->setExclusive( true );
 }
 
 JumpButtonBar::~JumpButtonBar()
@@ -85,27 +111,22 @@ JumpButtonBar::~JumpButtonBar()
 
 void JumpButtonBar::updateButtons()
 {
-  int currentButton = mGroupBox->selectedId();
+  mButtonsUpdated = true;
+
+  int currentButton = mButtonGroup->checkedId();
 
   // the easiest way to remove all buttons ;)
-  mButtons.setAutoDelete( true );
+  qDeleteAll(mButtons);
   mButtons.clear();
-  mButtons.setAutoDelete( false );
 
   QStringList characters;
 
   // calculate how many buttons are possible
-  QFontMetrics fm = fontMetrics();
-  QPushButton *btn = new QPushButton( "", this );
-  btn->hide();
-  QSize buttonSize = style().sizeFromContents( QStyle::CT_PushButton, btn,
-                     fm.size( ShowPrefix, "X - X") ).
-                     expandedTo( QApplication::globalStrut() );
-  delete btn;
 
-  int buttonHeight = buttonSize.height() + 8;
-  uint possibleButtons = (height() / buttonHeight) - 1;
+  int buttonHeight = calculateButtonSize( this ).height() + 8;
+  int possibleButtons = (height() / buttonHeight) - 1;
 
+  // collect all characters that could be appear
   QString character;
   KABC::AddressBook *ab = mCore->addressBook();
   KABC::AddressBook::Iterator it;
@@ -115,7 +136,7 @@ void JumpButtonBar::updateButtons()
     if ( field ) {
       setEnabled( true );
       if ( !field->value( *it ).isEmpty() )
-        character = field->value( *it )[ 0 ].lower();
+        character = field->value( *it )[ 0 ].toLower();
     } else {
       setEnabled( false );
       return;
@@ -125,35 +146,43 @@ void JumpButtonBar::updateButtons()
       characters.append( character );
   }
 
-  sortListLocaleAware( characters );
+  // sort the collected characters locale aware
+  qSort( characters.begin(), characters.end(), localAwareLessThan );
 
+  // create a new button for every character
   if ( characters.count() <= possibleButtons ) {
-    // at first the easy case: all buttons fits in window
-    for ( uint i = 0; i < characters.count(); ++i ) {
-      JumpButton *button = new JumpButton( characters[ i ], QString::null,
-                                           mGroupBox );
+    // at first the easy case: all buttons fits in window, so we assign one character per button
+    for ( int i = 0; i < characters.count(); ++i ) {
+      JumpButton *button = new JumpButton( characters[ i ], QString(),
+                                           this );
+      mLayout->addWidget( button );
+      mButtonGroup->addButton( button, mButtonGroup->buttons().size() );
       connect( button, SIGNAL( clicked() ), this, SLOT( letterClicked() ) );
       mButtons.append( button );
-      button->show();
     }
   } else {
+    // there is not enough space to put every character on its own button,
+    // so we create sequences (e.g. 'A - C', 'D - F')
+
     if ( possibleButtons == 0 ) // to avoid crashes on startup
       return;
+
     int offset = characters.count() / possibleButtons;
     int odd = characters.count() % possibleButtons;
     if ( odd )
       offset++;
 
     int current = 0;
-    for ( uint i = 0; i < possibleButtons; ++i ) {
+    for ( int i = 0; i < possibleButtons; ++i ) {
       if ( characters.count() - current == 0 )
         continue;
       if ( characters.count() - current <= possibleButtons - i ) {
         JumpButton *button = new JumpButton( characters[ current ],
-                                             QString::null, mGroupBox );
+                                             QString(), this );
+        mLayout->addWidget( button );
+        mButtonGroup->addButton( button, mButtonGroup->buttons().size() );
         connect( button, SIGNAL( clicked() ), this, SLOT( letterClicked() ) );
         mButtons.append( button );
-        button->show();
         current++;
       } else {
         int pos = ( current + offset >= (int)characters.count() ?
@@ -161,27 +190,24 @@ void JumpButtonBar::updateButtons()
         QString range;
         for ( int j = current; j < pos + 1; ++j )
           range.append( characters[ j ] );
+
         JumpButton *button = new JumpButton( characters[ current ],
-                                             characters[ pos ], mGroupBox );
+                                             characters[ pos ], this );
+        mLayout->addWidget( button );
+        mButtonGroup->addButton( button, mButtonGroup->buttons().size() );
         connect( button, SIGNAL( clicked() ), this, SLOT( letterClicked() ) );
         mButtons.append( button );
-        button->show();
         current = ( i + 1 ) * offset;
       }
     }
   }
 
   if ( currentButton != -1 )
-    mGroupBox->setButton( currentButton );
-  else
-    mGroupBox->setButton( 0 );
+    mButtonGroup->button( currentButton )->setChecked( true );
+  else if ( mButtonGroup->buttons().size() )
+    mButtonGroup->button( 0 )->setChecked( true );
 
-  int maxWidth = 0;
-  QPushButton *button;
-  for ( button = mButtons.first(); button; button = mButtons.next() )
-    maxWidth = QMAX( maxWidth, button->sizeHint().width() );
-
-  setFixedWidth( maxWidth );
+  mLayout->activate();
 }
 
 void JumpButtonBar::letterClicked()
@@ -194,46 +220,10 @@ void JumpButtonBar::letterClicked()
 
 void JumpButtonBar::resizeEvent( QResizeEvent* )
 {
-  updateButtons();
-}
-
-class SortContainer
-{
-  public:
-    SortContainer() {}
-    SortContainer( const QString &string )
-      : mString( string )
-    {
-    }
-
-    bool operator< ( const SortContainer &cnt )
-    {
-      return ( QString::localeAwareCompare( mString, cnt.mString ) < 0 );
-    }
-
-    QString data() const
-    {
-      return mString;
-    }
-
-  private:
-    QString mString;
-};
-
-void JumpButtonBar::sortListLocaleAware( QStringList &list )
-{
-  QValueList<SortContainer> sortList;
-
-  QStringList::ConstIterator it;
-  for ( it = list.begin(); it != list.end(); ++it )
-    sortList.append( SortContainer( *it ) );
-
-  qHeapSort( sortList );
-  list.clear();
-
-  QValueList<SortContainer>::ConstIterator sortIt;
-  for ( sortIt = sortList.begin(); sortIt != sortList.end(); ++sortIt )
-    list.append( (*sortIt).data() );
+  if ( mButtonsUpdated ) {
+    mButtonsUpdated = false;
+    QTimer::singleShot( 0, this, SLOT( updateButtons() ) );
+  }
 }
 
 #include "jumpbuttonbar.moc"
